@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2006-2009, KATO Noriaki
+ * Copyright (c) 2006-2010, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,17 +27,20 @@
  * \brief アルゴリズム
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2007-06-20
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.2/cstl/algorithm.h $
- * $Id: algorithm.h 264 2009-04-17 02:29:36Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.7.0/cstl/algorithm.h $
+ * $Id: algorithm.h 328 2010-01-24 03:49:15Z katono $
  */
 #ifndef CSTL_ALGORITHM_H_INCLUDED
 #define CSTL_ALGORITHM_H_INCLUDED
 
+#include <stdlib.h>
+#include "common.h"
 
-#define CSTL_ALGORITHM_SWAP(i, j, tmp, DIRECT_ACCESS)	\
-	tmp = DIRECT_ACCESS(self, i);\
-	DIRECT_ACCESS(self, i) = DIRECT_ACCESS(self, j);\
-	DIRECT_ACCESS(self, j) = tmp;\
+
+#define CSTL_ALGORITHM_SWAP(i, j, tmp)	\
+	tmp = *i;\
+	*i = *j;\
+	*j = tmp;\
 
 #define CSTL_ALGORITHM_SWITCH_INSERTION_SORT	(9)
 
@@ -71,9 +74,11 @@ static void Name##_insertion_sort(Name *self, size_t idx, size_t n, int (*comp)(
 {\
 	register size_t i, j;\
 	Type tmp;\
+	Type *alias1;\
+	Type *alias2;\
 	for (i = 1; i < n; i++) {\
-		for (j = i; j > 0 && comp(&DIRECT_ACCESS(self, idx + j - 1), &DIRECT_ACCESS(self, idx + j)) > 0; j--) {\
-			CSTL_ALGORITHM_SWAP(idx + j, idx + j - 1, tmp, DIRECT_ACCESS);\
+		for (j = i; j > 0 && comp(alias1 = &DIRECT_ACCESS(self, idx + j - 1), alias2 = &DIRECT_ACCESS(self, idx + j)) > 0; j--) {\
+			CSTL_ALGORITHM_SWAP(alias2, alias1, tmp);\
 		}\
 	}\
 }\
@@ -87,12 +92,12 @@ void Name##_sort(Name *self, size_t idx, size_t n, int (*comp)(const void *, con
 	register size_t sp;\
 	register size_t i, j;\
 	Type tmp;\
-	assert(self && "sort");\
-	assert(self->magic == self && "sort");\
-	assert(Name##_size(self) >= idx + n && "sort");\
-	assert(Name##_size(self) >= n && "sort");\
-	assert(Name##_size(self) > idx && "sort");\
-	assert(comp && "sort");\
+	CSTL_ASSERT(self && "sort");\
+	CSTL_ASSERT(self->magic == self && "sort");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "sort");\
+	CSTL_ASSERT(Name##_size(self) >= n && "sort");\
+	CSTL_ASSERT(Name##_size(self) > idx && "sort");\
+	CSTL_ASSERT(comp && "sort");\
 	low[0] = idx;\
 	high[0] = idx + n - 1;\
 	sp = 1;\
@@ -102,23 +107,30 @@ void Name##_sort(Name *self, size_t idx, size_t n, int (*comp)(const void *, con
 		l = low[sp];\
 		r = high[sp];\
 		if (l < r) {\
+			Type *alias1;\
+			Type *alias2;\
+			Type *alias3;\
 			if (r - l < CSTL_ALGORITHM_SWITCH_INSERTION_SORT) {\
 				Name##_insertion_sort(self, l, r - l + 1, comp);\
 				continue;\
 			}\
 			/* med3 */\
 			middle = (l + r) / 2;\
-			if (comp(&DIRECT_ACCESS(self, l), &DIRECT_ACCESS(self, middle)) > 0) {\
-				CSTL_ALGORITHM_SWAP(l, middle, tmp, DIRECT_ACCESS);\
+			alias1 = &DIRECT_ACCESS(self, l);\
+			alias2 = &DIRECT_ACCESS(self, middle);\
+			alias3 = &DIRECT_ACCESS(self, r);\
+			if (comp(alias1, alias2) > 0) {\
+				CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 			}\
-			if (comp(&DIRECT_ACCESS(self, l), &DIRECT_ACCESS(self, r)) > 0) {\
-				CSTL_ALGORITHM_SWAP(l, r, tmp, DIRECT_ACCESS);\
+			if (comp(alias1, alias3) > 0) {\
+				CSTL_ALGORITHM_SWAP(alias1, alias3, tmp);\
 			}\
-			if (comp(&DIRECT_ACCESS(self, middle), &DIRECT_ACCESS(self, r)) > 0) {\
-				CSTL_ALGORITHM_SWAP(middle, r, tmp, DIRECT_ACCESS);\
+			if (comp(alias2, alias3) > 0) {\
+				CSTL_ALGORITHM_SWAP(alias2, alias3, tmp);\
 			}\
 			/* r - 1の要素をを枢軸にする */\
-			CSTL_ALGORITHM_SWAP(middle, r - 1, tmp, DIRECT_ACCESS);\
+			alias3 = &DIRECT_ACCESS(self, r - 1);\
+			CSTL_ALGORITHM_SWAP(alias2, alias3, tmp);\
 			/* partition */\
 			i = l;\
 			j = r - 1;\
@@ -132,9 +144,12 @@ void Name##_sort(Name *self, size_t idx, size_t n, int (*comp)(const void *, con
 				if (i >= j) {\
 					break;\
 				}\
-				CSTL_ALGORITHM_SWAP(i, j, tmp, DIRECT_ACCESS);\
+				alias1 = &DIRECT_ACCESS(self, i);\
+				alias2 = &DIRECT_ACCESS(self, j);\
+				CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 			}\
-			CSTL_ALGORITHM_SWAP(i, r - 1, tmp, DIRECT_ACCESS);\
+			alias1 = &DIRECT_ACCESS(self, i);\
+			CSTL_ALGORITHM_SWAP(alias1, alias3, tmp);\
 			/* stack push */\
 			if (i - l < r - i) {\
 				low[sp] = i + 1;\
@@ -182,7 +197,9 @@ static size_t Name##_rotate_aux(Name *self, size_t first, size_t middle, size_t
 		return last;\
 	} else if (k == l) {\
 		for (i = 0; i < k; i++) {\
-			CSTL_ALGORITHM_SWAP(first + i, middle + i, tmp, DIRECT_ACCESS);\
+			Type *alias1 = &DIRECT_ACCESS(self, first + i);\
+			Type *alias2 = &DIRECT_ACCESS(self, middle + i);\
+			CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 		}\
 		return result;\
 	}\
@@ -231,8 +248,12 @@ static void Name##_merge_without_buffer(Name *self, size_t first, size_t middle,
 		return;\
 	}\
 	if (len1 + len2 == 2) {\
-		if (comp(&DIRECT_ACCESS(self, first), &DIRECT_ACCESS(self, middle)) > 0) {\
-			CSTL_ALGORITHM_SWAP(first, middle, tmp, DIRECT_ACCESS);\
+		Type *alias1;\
+		Type *alias2;\
+		alias1 = &DIRECT_ACCESS(self, first);\
+		alias2 = &DIRECT_ACCESS(self, middle);\
+		if (comp(alias1, alias2) > 0) {\
+			CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 		}\
 		return;\
 	}\
@@ -301,12 +322,12 @@ static void Name##_merge_sort(Name *self, size_t first, size_t last, Type *buf,
 void Name##_stable_sort(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *))\
 {\
 	Type *buf;\
-	assert(self && "stable_sort");\
-	assert(self->magic == self && "stable_sort");\
-	assert(Name##_size(self) >= idx + n && "stable_sort");\
-	assert(Name##_size(self) >= n && "stable_sort");\
-	assert(Name##_size(self) > idx && "stable_sort");\
-	assert(comp && "stable_sort");\
+	CSTL_ASSERT(self && "stable_sort");\
+	CSTL_ASSERT(self->magic == self && "stable_sort");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "stable_sort");\
+	CSTL_ASSERT(Name##_size(self) >= n && "stable_sort");\
+	CSTL_ASSERT(Name##_size(self) > idx && "stable_sort");\
+	CSTL_ASSERT(comp && "stable_sort");\
 	if (n < CSTL_ALGORITHM_SWITCH_INSERTION_SORT) {\
 		Name##_insertion_sort(self, idx, n, comp);\
 		return;\
@@ -321,12 +342,12 @@ size_t Name##_lower_bound(Name *self, size_t idx, size_t n, Type value, int (*co
 	register size_t first;\
 	register size_t last;\
 	register size_t middle;\
-	assert(self && "lower_bound");\
-	assert(self->magic == self && "lower_bound");\
-	assert(Name##_size(self) >= idx + n && "lower_bound");\
-	assert(Name##_size(self) >= n && "lower_bound");\
-	assert(Name##_size(self) > idx && "lower_bound");\
-	assert(comp && "lower_bound");\
+	CSTL_ASSERT(self && "lower_bound");\
+	CSTL_ASSERT(self->magic == self && "lower_bound");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "lower_bound");\
+	CSTL_ASSERT(Name##_size(self) >= n && "lower_bound");\
+	CSTL_ASSERT(Name##_size(self) > idx && "lower_bound");\
+	CSTL_ASSERT(comp && "lower_bound");\
 	first = idx;\
 	last = idx + n;\
 	while (first < last) {\
@@ -345,12 +366,12 @@ size_t Name##_upper_bound(Name *self, size_t idx, size_t n, Type value, int (*co
 	register size_t first;\
 	register size_t last;\
 	register size_t middle;\
-	assert(self && "upper_bound");\
-	assert(self->magic == self && "upper_bound");\
-	assert(Name##_size(self) >= idx + n && "upper_bound");\
-	assert(Name##_size(self) >= n && "upper_bound");\
-	assert(Name##_size(self) > idx && "upper_bound");\
-	assert(comp && "upper_bound");\
+	CSTL_ASSERT(self && "upper_bound");\
+	CSTL_ASSERT(self->magic == self && "upper_bound");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "upper_bound");\
+	CSTL_ASSERT(Name##_size(self) >= n && "upper_bound");\
+	CSTL_ASSERT(Name##_size(self) > idx && "upper_bound");\
+	CSTL_ASSERT(comp && "upper_bound");\
 	first = idx;\
 	last = idx + n;\
 	while (first < last) {\
@@ -367,12 +388,12 @@ size_t Name##_upper_bound(Name *self, size_t idx, size_t n, Type value, int (*co
 size_t Name##_binary_search(Name *self, size_t idx, size_t n, Type value, int (*comp)(const void *, const void *))\
 {\
 	size_t i;\
-	assert(self && "binary_search");\
-	assert(self->magic == self && "binary_search");\
-	assert(Name##_size(self) >= idx + n && "binary_search");\
-	assert(Name##_size(self) >= n && "binary_search");\
-	assert(Name##_size(self) > idx && "binary_search");\
-	assert(comp && "binary_search");\
+	CSTL_ASSERT(self && "binary_search");\
+	CSTL_ASSERT(self->magic == self && "binary_search");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "binary_search");\
+	CSTL_ASSERT(Name##_size(self) >= n && "binary_search");\
+	CSTL_ASSERT(Name##_size(self) > idx && "binary_search");\
+	CSTL_ASSERT(comp && "binary_search");\
 	i = Name##_lower_bound(self, idx, n, value, comp);\
 	if (i == idx + n) {\
 		return i;\
@@ -388,15 +409,17 @@ void Name##_reverse(Name *self, size_t idx, size_t n)\
 	register size_t first;\
 	register size_t last;\
 	Type tmp;\
-	assert(self && "reverse");\
-	assert(self->magic == self && "reverse");\
-	assert(Name##_size(self) >= idx + n && "reverse");\
-	assert(Name##_size(self) >= n && "reverse");\
-	assert(Name##_size(self) > idx && "reverse");\
+	CSTL_ASSERT(self && "reverse");\
+	CSTL_ASSERT(self->magic == self && "reverse");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "reverse");\
+	CSTL_ASSERT(Name##_size(self) >= n && "reverse");\
+	CSTL_ASSERT(Name##_size(self) > idx && "reverse");\
 	first = idx;\
 	last = idx + n - 1;\
 	while (first < last) {\
-		CSTL_ALGORITHM_SWAP(first, last, tmp, DIRECT_ACCESS);\
+		Type *alias1 = &DIRECT_ACCESS(self, first);\
+		Type *alias2 = &DIRECT_ACCESS(self, last);\
+		CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 		first++;\
 		last--;\
 	}\
@@ -404,11 +427,11 @@ void Name##_reverse(Name *self, size_t idx, size_t n)\
 \
 void Name##_rotate(Name *self, size_t first, size_t middle, size_t last)\
 {\
-	assert(self && "rotate");\
-	assert(self->magic == self && "rotate");\
-	assert(first <= middle && "rotate");\
-	assert(middle <= last && "rotate");\
-	assert(Name##_size(self) >= last && "rotate");\
+	CSTL_ASSERT(self && "rotate");\
+	CSTL_ASSERT(self->magic == self && "rotate");\
+	CSTL_ASSERT(first <= middle && "rotate");\
+	CSTL_ASSERT(middle <= last && "rotate");\
+	CSTL_ASSERT(Name##_size(self) >= last && "rotate");\
 	Name##_rotate_aux(self, first, middle, last);\
 }\
 \
@@ -416,32 +439,34 @@ int Name##_merge(Name *self, size_t idx, \
 		Name *x, size_t xidx, size_t xn, Name *y, size_t yidx, size_t yn, int (*comp)(const void *, const void *))\
 {\
 	register size_t i, j, k;\
-	assert(self && "merge");\
-	assert(self->magic == self && "merge");\
-	assert(Name##_size(self) >= idx && "merge");\
-	assert(self != x && "merge");\
-	assert(self != y && "merge");\
-	assert(x && "merge");\
-	assert(x->magic == x && "merge");\
-	assert(Name##_size(x) >= xidx + xn && "merge");\
-	assert(Name##_size(x) >= xn && "merge");\
-	assert(Name##_size(x) > xidx && "merge");\
-	assert(y && "merge");\
-	assert(y->magic == y && "merge");\
-	assert(Name##_size(y) >= yidx + yn && "merge");\
-	assert(Name##_size(y) >= yn && "merge");\
-	assert(Name##_size(y) > yidx && "merge");\
-	assert(comp && "merge");\
+	CSTL_ASSERT(self && "merge");\
+	CSTL_ASSERT(self->magic == self && "merge");\
+	CSTL_ASSERT(Name##_size(self) >= idx && "merge");\
+	CSTL_ASSERT(self != x && "merge");\
+	CSTL_ASSERT(self != y && "merge");\
+	CSTL_ASSERT(x && "merge");\
+	CSTL_ASSERT(x->magic == x && "merge");\
+	CSTL_ASSERT(Name##_size(x) >= xidx + xn && "merge");\
+	CSTL_ASSERT(Name##_size(x) >= xn && "merge");\
+	CSTL_ASSERT(Name##_size(x) > xidx && "merge");\
+	CSTL_ASSERT(y && "merge");\
+	CSTL_ASSERT(y->magic == y && "merge");\
+	CSTL_ASSERT(Name##_size(y) >= yidx + yn && "merge");\
+	CSTL_ASSERT(Name##_size(y) >= yn && "merge");\
+	CSTL_ASSERT(Name##_size(y) > yidx && "merge");\
+	CSTL_ASSERT(comp && "merge");\
 	if (!Name##_insert_n_no_data(self, idx, xn + yn)) {\
 		return 0;\
 	}\
 	i = j = k = 0;\
 	while (i < xn && j < yn) {\
-		if (comp(&DIRECT_ACCESS(x, xidx + i), &DIRECT_ACCESS(y, yidx + j)) <= 0) {\
-			DIRECT_ACCESS(self, idx + k) = DIRECT_ACCESS(x, xidx + i);\
+		Type *alias1;\
+		Type *alias2;\
+		if (comp(alias1 = &DIRECT_ACCESS(x, xidx + i), alias2 = &DIRECT_ACCESS(y, yidx + j)) <= 0) {\
+			DIRECT_ACCESS(self, idx + k) = *alias1;\
 			i++;\
 		} else {\
-			DIRECT_ACCESS(self, idx + k) = DIRECT_ACCESS(y, yidx + j);\
+			DIRECT_ACCESS(self, idx + k) = *alias2;\
 			j++;\
 		}\
 		k++;\
@@ -465,12 +490,12 @@ int Name##_merge(Name *self, size_t idx, \
 void Name##_inplace_merge(Name *self, size_t first, size_t middle, size_t last, int (*comp)(const void *, const void *))\
 {\
 	Type *buf;\
-	assert(self && "inplace_merge");\
-	assert(self->magic == self && "inplace_merge");\
-	assert(first <= middle && "inplace_merge");\
-	assert(middle <= last && "inplace_merge");\
-	assert(Name##_size(self) >= last && "inplace_merge");\
-	assert(comp && "inplace_merge");\
+	CSTL_ASSERT(self && "inplace_merge");\
+	CSTL_ASSERT(self->magic == self && "inplace_merge");\
+	CSTL_ASSERT(first <= middle && "inplace_merge");\
+	CSTL_ASSERT(middle <= last && "inplace_merge");\
+	CSTL_ASSERT(Name##_size(self) >= last && "inplace_merge");\
+	CSTL_ASSERT(comp && "inplace_merge");\
 	if (first == middle || middle == last) {\
 		return;\
 	}\
@@ -488,10 +513,11 @@ static void Name##_up_heap(Name *self, size_t top_idx, size_t hi_idx, int (*comp
 	/* hi_の付く変数は1から始まるヒープのインデックスを示す */\
 	register size_t hi_i;\
 	Type tmp;\
+	Type *alias;\
 	hi_i = hi_idx;\
 	tmp = DIRECT_ACCESS(self, hi_i + top_idx - 1);\
-	while (hi_i > 1 && comp(&DIRECT_ACCESS(self, hi_i / 2 + top_idx - 1), &tmp) < 0) {\
-		DIRECT_ACCESS(self, hi_i + top_idx - 1) = DIRECT_ACCESS(self, hi_i / 2 + top_idx - 1);\
+	while (hi_i > 1 && comp(alias = &DIRECT_ACCESS(self, hi_i / 2 + top_idx - 1), &tmp) < 0) {\
+		DIRECT_ACCESS(self, hi_i + top_idx - 1) = *alias;\
 		hi_i = hi_i / 2;\
 	}\
 	DIRECT_ACCESS(self, hi_i + top_idx - 1) = tmp;\
@@ -502,6 +528,7 @@ static void Name##_down_heap(Name *self, size_t top_idx, size_t hi_from, size_t
 	/* hi_の付く変数は1から始まるヒープのインデックスを示す */\
 	register size_t hi_i, hi_j;\
 	Type tmp;\
+	Type *alias;\
 	hi_j = hi_from;\
 	tmp = DIRECT_ACCESS(self, hi_j + top_idx - 1);\
 	while (2 * hi_j <= hi_to) {\
@@ -510,8 +537,8 @@ static void Name##_down_heap(Name *self, size_t top_idx, size_t hi_from, size_t
 			/* 右の子が存在し、左より右の子が大きい */\
 			hi_i++;\
 		}\
-		if (comp(&tmp, &DIRECT_ACCESS(self, hi_i + top_idx - 1)) < 0) {\
-			DIRECT_ACCESS(self, hi_j + top_idx - 1) = DIRECT_ACCESS(self, hi_i + top_idx - 1);\
+		if (comp(&tmp, alias = &DIRECT_ACCESS(self, hi_i + top_idx - 1)) < 0) {\
+			DIRECT_ACCESS(self, hi_j + top_idx - 1) = *alias;\
 			hi_j = hi_i;\
 		} else {\
 			break;\
@@ -522,38 +549,42 @@ static void Name##_down_heap(Name *self, size_t top_idx, size_t hi_from, size_t
 \
 void Name##_push_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *))\
 {\
-	assert(self && "push_heap");\
-	assert(self->magic == self && "push_heap");\
-	assert(Name##_size(self) >= idx + n && "push_heap");\
-	assert(Name##_size(self) >= n && "push_heap");\
-	assert(Name##_size(self) > idx && "push_heap");\
-	assert(comp && "push_heap");\
+	CSTL_ASSERT(self && "push_heap");\
+	CSTL_ASSERT(self->magic == self && "push_heap");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "push_heap");\
+	CSTL_ASSERT(Name##_size(self) >= n && "push_heap");\
+	CSTL_ASSERT(Name##_size(self) > idx && "push_heap");\
+	CSTL_ASSERT(comp && "push_heap");\
 	Name##_up_heap(self, idx, n, comp);\
 }\
 \
 void Name##_pop_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *))\
 {\
 	Type tmp;\
-	assert(self && "pop_heap");\
-	assert(self->magic == self && "pop_heap");\
-	assert(Name##_size(self) >= idx + n && "pop_heap");\
-	assert(Name##_size(self) >= n && "pop_heap");\
-	assert(Name##_size(self) > idx && "pop_heap");\
-	assert(comp && "pop_heap");\
-	assert(n > 0 && "pop_heap");\
-	CSTL_ALGORITHM_SWAP(idx, idx + n - 1, tmp, DIRECT_ACCESS);\
+	Type *alias1;\
+	Type *alias2;\
+	CSTL_ASSERT(self && "pop_heap");\
+	CSTL_ASSERT(self->magic == self && "pop_heap");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "pop_heap");\
+	CSTL_ASSERT(Name##_size(self) >= n && "pop_heap");\
+	CSTL_ASSERT(Name##_size(self) > idx && "pop_heap");\
+	CSTL_ASSERT(comp && "pop_heap");\
+	CSTL_ASSERT(n > 0 && "pop_heap");\
+	alias1 = &DIRECT_ACCESS(self, idx);\
+	alias2 = &DIRECT_ACCESS(self, idx + n - 1);\
+	CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 	Name##_down_heap(self, idx, 1, n - 1, comp);\
 }\
 \
 void Name##_make_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *, const void *))\
 {\
 	register size_t i;\
-	assert(self && "make_heap");\
-	assert(self->magic == self && "make_heap");\
-	assert(Name##_size(self) >= idx + n && "make_heap");\
-	assert(Name##_size(self) >= n && "make_heap");\
-	assert(Name##_size(self) > idx && "make_heap");\
-	assert(comp && "make_heap");\
+	CSTL_ASSERT(self && "make_heap");\
+	CSTL_ASSERT(self->magic == self && "make_heap");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "make_heap");\
+	CSTL_ASSERT(Name##_size(self) >= n && "make_heap");\
+	CSTL_ASSERT(Name##_size(self) > idx && "make_heap");\
+	CSTL_ASSERT(comp && "make_heap");\
 	for (i = n / 2; i > 0; i--) {\
 		Name##_down_heap(self, idx, i, n, comp);\
 	}\
@@ -563,14 +594,16 @@ void Name##_sort_heap(Name *self, size_t idx, size_t n, int (*comp)(const void *
 {\
 	register size_t i;\
 	Type tmp;\
-	assert(self && "sort_heap");\
-	assert(self->magic == self && "sort_heap");\
-	assert(Name##_size(self) >= idx + n && "sort_heap");\
-	assert(Name##_size(self) >= n && "sort_heap");\
-	assert(Name##_size(self) > idx && "sort_heap");\
-	assert(comp && "sort_heap");\
+	CSTL_ASSERT(self && "sort_heap");\
+	CSTL_ASSERT(self->magic == self && "sort_heap");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "sort_heap");\
+	CSTL_ASSERT(Name##_size(self) >= n && "sort_heap");\
+	CSTL_ASSERT(Name##_size(self) > idx && "sort_heap");\
+	CSTL_ASSERT(comp && "sort_heap");\
 	for (i = n; i > 1; i--) {\
-		CSTL_ALGORITHM_SWAP(idx, idx + i - 1, tmp, DIRECT_ACCESS);\
+		Type *alias1 = &DIRECT_ACCESS(self, idx);\
+		Type *alias2 = &DIRECT_ACCESS(self, idx + i - 1);\
+		CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 		Name##_down_heap(self, idx, 1, i - 1, comp);\
 	}\
 }\
@@ -579,18 +612,22 @@ void Name##_partial_sort(Name *self, size_t idx, size_t sort_n, size_t n, int (*
 {\
 	register size_t i;\
 	Type tmp;\
-	assert(self && "partial_sort");\
-	assert(self->magic == self && "partial_sort");\
-	assert(Name##_size(self) >= idx + n && "partial_sort");\
-	assert(Name##_size(self) >= n && "partial_sort");\
-	assert(Name##_size(self) > idx && "partial_sort");\
-	assert(sort_n <= n && "partial_sort");\
-	assert(comp && "partial_sort");\
+	CSTL_ASSERT(self && "partial_sort");\
+	CSTL_ASSERT(self->magic == self && "partial_sort");\
+	CSTL_ASSERT(Name##_size(self) >= idx + n && "partial_sort");\
+	CSTL_ASSERT(Name##_size(self) >= n && "partial_sort");\
+	CSTL_ASSERT(Name##_size(self) > idx && "partial_sort");\
+	CSTL_ASSERT(sort_n <= n && "partial_sort");\
+	CSTL_ASSERT(comp && "partial_sort");\
 	Name##_make_heap(self, idx, sort_n, comp);\
 	for (i = idx + sort_n; i < idx + n; i++) {\
 		if (comp(&DIRECT_ACCESS(self, idx), &DIRECT_ACCESS(self, i)) > 0) {\
+			Type *alias1;\
+			Type *alias2;\
 			Name##_pop_heap(self, idx, sort_n, comp);\
-			CSTL_ALGORITHM_SWAP(idx + sort_n - 1, i, tmp, DIRECT_ACCESS);\
+			alias1 = &DIRECT_ACCESS(self, idx + sort_n - 1);\
+			alias2 = &DIRECT_ACCESS(self, i);\
+			CSTL_ALGORITHM_SWAP(alias1, alias2, tmp);\
 			Name##_push_heap(self, idx, sort_n, comp);\
 		}\
 	}\
