@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2006-2009, KATO Noriaki
+ * Copyright (c) 2006-2010, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,8 +27,8 @@
  * \brief 赤黒木
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2006-10-29
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.2/cstl/rbtree.h $
- * $Id: rbtree.h 278 2009-05-19 06:43:20Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.7.0/cstl/rbtree.h $
+ * $Id: rbtree.h 328 2010-01-24 03:49:15Z katono $
  *
  * このファイルを直接インクルードしないこと
  */
@@ -36,39 +36,34 @@
 #define CSTL_RBTREE_H_INCLUDED
 
 #include <stdlib.h>
-#include <assert.h>
+#include "common.h"
 
-#ifdef NDEBUG
-#define CSTL_RBTREE_MAGIC(x)
-#else
-#define CSTL_RBTREE_MAGIC(x) x
-#endif
-
-/*! 
- * \brief 赤黒木の色
- */
-enum {
-	CSTL_RBTREE_RED,
-	CSTL_RBTREE_BLACK,
-	CSTL_RBTREE_HEAD
-};
 
 #define CSTL_LESS(x, y)		((x) == (y) ? 0 : (x) < (y) ? -1 : 1)
 #define CSTL_GREATER(x, y)	((x) == (y) ? 0 : (x) > (y) ? -1 : 1)
 
 
-#define CSTL_RBTREE_IS_HEAD(node)		((node)->color == CSTL_RBTREE_HEAD)
-#define CSTL_RBTREE_IS_ROOT(node)		CSTL_RBTREE_IS_HEAD((node)->parent)
-#define CSTL_RBTREE_IS_NIL(node, Name)	((node) == (Name##RBTree *) &Name##RBTree_nil)
+#define CSTL_RBTREE_IS_HEAD(node, Name)		((node)->color == Name##_COLOR_HEAD)
+#define CSTL_RBTREE_IS_ROOT(node, Name)		CSTL_RBTREE_IS_HEAD((node)->parent, Name)
+#define CSTL_RBTREE_IS_NIL(node, Name)		((node) == (Name##RBTree *) &Name##RBTree_nil)
 
 
 #define CSTL_RBTREE_IMPLEMENT(Name, KeyType, ValueType, Compare)	\
 \
+/*! \
+ * \brief 赤黒木の色\
+ */\
+enum {\
+	Name##_COLOR_RED,\
+	Name##_COLOR_BLACK,\
+	Name##_COLOR_HEAD\
+};\
+\
 static const Name##RBTree Name##RBTree_nil = {\
 	(Name##RBTree *) &Name##RBTree_nil, \
 	(Name##RBTree *) &Name##RBTree_nil, \
 	(Name##RBTree *) &Name##RBTree_nil, \
-	CSTL_RBTREE_BLACK\
+	Name##_COLOR_BLACK\
 };\
 \
 static Name##RBTree *Name##RBTree_new(void);\
@@ -106,7 +101,7 @@ static void Name##RBTree_balance_for_erase(Name##RBTree *n, Name##RBTree *p_of_n
 \
 static void Name##RBTree_set_left(Name##RBTree *node, Name##RBTree *t)\
 {\
-	assert(node && "RBTree_set_left");\
+	CSTL_ASSERT(node && "RBTree_set_left");\
 	node->left = t;\
 	if (!CSTL_RBTREE_IS_NIL(t, Name)) {\
 		t->parent = node;\
@@ -115,7 +110,7 @@ static void Name##RBTree_set_left(Name##RBTree *node, Name##RBTree *t)\
 \
 static void Name##RBTree_set_right(Name##RBTree *node, Name##RBTree *t)\
 {\
-	assert(node && "RBTree_set_right");\
+	CSTL_ASSERT(node && "RBTree_set_right");\
 	node->right = t;\
 	if (!CSTL_RBTREE_IS_NIL(t, Name)) {\
 		t->parent = node;\
@@ -124,13 +119,13 @@ static void Name##RBTree_set_right(Name##RBTree *node, Name##RBTree *t)\
 \
 static Name##RBTree *Name##RBTree_get_root(Name##RBTree *self)\
 {\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_get_root");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_get_root");\
 	return self->right;\
 }\
 \
 static void Name##RBTree_set_root(Name##RBTree *self, Name##RBTree *t)\
 {\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_set_root");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_set_root");\
 	self->right = t;\
 	if (!CSTL_RBTREE_IS_NIL(t, Name)) {\
 		t->parent = self;\
@@ -145,8 +140,8 @@ static Name##RBTree *Name##RBTree_new(void)\
 	self->left = (Name##RBTree *) &Name##RBTree_nil;\
 	self->right = (Name##RBTree *) &Name##RBTree_nil;\
 	self->parent = (Name##RBTree *) &Name##RBTree_nil;\
-	self->color = CSTL_RBTREE_HEAD;\
-	CSTL_RBTREE_MAGIC(self->magic = self);\
+	self->color = Name##_COLOR_HEAD;\
+	CSTL_MAGIC(self->magic = self);\
 	return self;\
 }\
 \
@@ -154,7 +149,7 @@ static void Name##RBTree_clear(Name##RBTree *self)\
 {\
 	register Name##RBTree *t;\
 	register Name##RBTree *tmp;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_clear");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_clear");\
 	t = Name##RBTree_get_root(self);\
 	if (CSTL_RBTREE_IS_NIL(t, Name)) return;\
 	while (1) {\
@@ -172,24 +167,24 @@ static void Name##RBTree_clear(Name##RBTree *self)\
 			t->parent->right = (Name##RBTree *) &Name##RBTree_nil;\
 		}\
 		tmp = t->parent;\
-		CSTL_RBTREE_MAGIC(t->magic = 0);\
+		CSTL_MAGIC(t->magic = 0);\
 		free(t);\
 		t = tmp;\
-		if (CSTL_RBTREE_IS_HEAD(t)) break;\
+		if (CSTL_RBTREE_IS_HEAD(t, Name)) break;\
 	}\
 }\
 \
 static void Name##RBTree_delete(Name##RBTree *self)\
 {\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_delete");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_delete");\
 	Name##RBTree_clear(self);\
-	CSTL_RBTREE_MAGIC(self->magic = 0);\
+	CSTL_MAGIC(self->magic = 0);\
 	free(self);\
 }\
 \
 static int Name##RBTree_empty(Name##RBTree *self)\
 {\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_empty");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_empty");\
 	return CSTL_RBTREE_IS_NIL(Name##RBTree_get_root(self), Name);\
 }\
 \
@@ -212,7 +207,7 @@ static Name##RBTree *Name##RBTree_find_node(Name##RBTree *t, KeyType key)\
 static Name##Iterator Name##RBTree_find(Name##RBTree *self, KeyType key)\
 {\
 	Name##RBTree *t;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_find");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_find");\
 	t = Name##RBTree_find_node(Name##RBTree_get_root(self), key);\
 	return CSTL_RBTREE_IS_NIL(t, Name) ? Name##RBTree_end(self) : t;\
 }\
@@ -223,7 +218,7 @@ static size_t Name##RBTree_count(Name##RBTree *self, KeyType key)\
 	register Name##Iterator pos;\
 	register Name##Iterator first;\
 	register Name##Iterator last;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_count");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_count");\
 	first = Name##RBTree_lower_bound(self, key);\
 	last = Name##RBTree_upper_bound(self, key);\
 	for (pos = first; pos != last; pos = Name##RBTree_next(pos)) {\
@@ -236,7 +231,7 @@ static Name##Iterator Name##RBTree_lower_bound(Name##RBTree *self, KeyType key)\
 {\
 	register Name##RBTree *t;\
 	register Name##RBTree *tmp;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_lower_bound");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_lower_bound");\
 	tmp = Name##RBTree_end(self);\
 	t = Name##RBTree_get_root(self);\
 	while (!CSTL_RBTREE_IS_NIL(t, Name)) {\
@@ -254,7 +249,7 @@ static Name##Iterator Name##RBTree_upper_bound(Name##RBTree *self, KeyType key)\
 {\
 	register Name##RBTree *t;\
 	register Name##RBTree *tmp;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_upper_bound");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_upper_bound");\
 	tmp = Name##RBTree_end(self);\
 	t = Name##RBTree_get_root(self);\
 	while (!CSTL_RBTREE_IS_NIL(t, Name)) {\
@@ -270,8 +265,8 @@ static Name##Iterator Name##RBTree_upper_bound(Name##RBTree *self, KeyType key)\
 \
 static Name##RBTree *Name##RBTree_replace_subtree(Name##RBTree *node, Name##RBTree *t)\
 {\
-	assert(!CSTL_RBTREE_IS_HEAD(node) && "RBTree_replace_subtree");\
-	assert(!CSTL_RBTREE_IS_HEAD(t) && "RBTree_replace_subtree");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(node, Name) && "RBTree_replace_subtree");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(t, Name) && "RBTree_replace_subtree");\
 	if (node->parent->left == node) {\
 		Name##RBTree_set_left(node->parent, t);\
 	} else {\
@@ -308,10 +303,10 @@ static void Name##RBTree_swap(Name##RBTree *s, Name##RBTree *t)\
 	Name##RBTree *tl;\
 	Name##RBTree *tr;\
 	int c;\
-	assert(!CSTL_RBTREE_IS_HEAD(s) && "RBTree_swap");\
-	assert(!CSTL_RBTREE_IS_HEAD(t) && "RBTree_swap");\
-	assert(!CSTL_RBTREE_IS_NIL(s, Name) && "RBTree_swap");\
-	assert(!CSTL_RBTREE_IS_NIL(t, Name) && "RBTree_swap");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(s, Name) && "RBTree_swap");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(t, Name) && "RBTree_swap");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(s, Name) && "RBTree_swap");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(t, Name) && "RBTree_swap");\
 	if (t->parent == s) {\
 		Name##RBTree_swap_parent_child(s, t);\
 	} else if (s->parent == t) {\
@@ -344,11 +339,11 @@ static void Name##RBTree_rotate_right(Name##RBTree *node)\
 {\
 	Name##RBTree *p;\
 	Name##RBTree *n;\
-	assert(!CSTL_RBTREE_IS_HEAD(node) && "RBTree_rotate_right");\
-	assert(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_rotate_right");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(node, Name) && "RBTree_rotate_right");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_rotate_right");\
 	p = node->parent;\
 	n = node->left;\
-	assert(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_rotate_right");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_rotate_right");\
 	Name##RBTree_set_left(node, n->right);\
 	Name##RBTree_set_right(n, node);\
 	if (p->left == node) {\
@@ -362,11 +357,11 @@ static void Name##RBTree_rotate_left(Name##RBTree *node)\
 {\
 	Name##RBTree *p;\
 	Name##RBTree *n;\
-	assert(!CSTL_RBTREE_IS_HEAD(node) && "RBTree_rotate_left");\
-	assert(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_rotate_left");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(node, Name) && "RBTree_rotate_left");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_rotate_left");\
 	p = node->parent;\
 	n = node->right;\
-	assert(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_rotate_left");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_rotate_left");\
 	Name##RBTree_set_right(node, n->left);\
 	Name##RBTree_set_left(n, node);\
 	if (p->left == node) {\
@@ -378,9 +373,9 @@ static void Name##RBTree_rotate_left(Name##RBTree *node)\
 \
 static Name##RBTree *Name##RBTree_get_sibling(Name##RBTree *node)\
 {\
-	assert(!CSTL_RBTREE_IS_HEAD(node) && "RBTree_get_sibling");\
-	assert(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_get_sibling");\
-	assert(!CSTL_RBTREE_IS_ROOT(node) && "RBTree_get_sibling");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(node, Name) && "RBTree_get_sibling");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_get_sibling");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_ROOT(node, Name) && "RBTree_get_sibling");\
 	return (node->parent->left == node) ? \
 		node->parent->right : node->parent->left;\
 }\
@@ -388,13 +383,13 @@ static Name##RBTree *Name##RBTree_get_sibling(Name##RBTree *node)\
 static Name##RBTree *Name##RBTree_get_uncle(Name##RBTree *node)\
 {\
 	Name##RBTree *g;\
-	assert(!CSTL_RBTREE_IS_HEAD(node) && "RBTree_get_uncle");\
-	assert(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_get_uncle");\
-	assert(!CSTL_RBTREE_IS_ROOT(node) && "RBTree_get_uncle");\
-	assert(!CSTL_RBTREE_IS_ROOT(node->parent) && "RBTree_get_uncle");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(node, Name) && "RBTree_get_uncle");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(node, Name) && "RBTree_get_uncle");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_ROOT(node, Name) && "RBTree_get_uncle");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_ROOT(node->parent, Name) && "RBTree_get_uncle");\
 	g = node->parent->parent;\
-	assert(!CSTL_RBTREE_IS_NIL(g, Name) && "RBTree_get_uncle");\
-	assert(!CSTL_RBTREE_IS_HEAD(g) && "RBTree_get_uncle");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(g, Name) && "RBTree_get_uncle");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(g, Name) && "RBTree_get_uncle");\
 	return (g->left == node->parent) ?\
 		g->right : g->left;\
 }\
@@ -407,23 +402,23 @@ static void Name##RBTree_balance_for_insert(Name##RBTree *n)\
 	Name##RBTree *tmp;\
 	while (1) {\
 		p = n->parent;\
-		if (CSTL_RBTREE_IS_ROOT(n)) {\
+		if (CSTL_RBTREE_IS_ROOT(n, Name)) {\
 			/* case 1 nがroot */\
-			n->color = CSTL_RBTREE_BLACK;\
+			n->color = Name##_COLOR_BLACK;\
 			break;\
 		}\
-		if (p->color == CSTL_RBTREE_BLACK) {\
+		if (p->color == Name##_COLOR_BLACK) {\
 			/* case 2 pが黒 */\
 			break;\
 		}\
 		/* 以下、pは赤、gは黒 */\
 		g = p->parent;\
 		u = Name##RBTree_get_uncle(n);\
-		if (u->color == CSTL_RBTREE_RED) {\
+		if (u->color == Name##_COLOR_RED) {\
 			/* case 3 uが赤 */\
-			g->color = CSTL_RBTREE_RED;\
-			p->color = CSTL_RBTREE_BLACK;\
-			u->color = CSTL_RBTREE_BLACK;\
+			g->color = Name##_COLOR_RED;\
+			p->color = Name##_COLOR_BLACK;\
+			u->color = Name##_COLOR_BLACK;\
 			n = g;\
 			/* gをnにして再調整。最悪の場合、根まで伝播 */\
 			continue;\
@@ -451,10 +446,10 @@ static void Name##RBTree_balance_for_insert(Name##RBTree *n)\
 			/* case 5 right */\
 			Name##RBTree_rotate_left(g);\
 		} else {\
-			assert(0 && "RBTree_balance_for_insert");\
+			CSTL_ASSERT(0 && "RBTree_balance_for_insert");\
 		}\
-		p->color = CSTL_RBTREE_BLACK;\
-		g->color = CSTL_RBTREE_RED;\
+		p->color = Name##_COLOR_BLACK;\
+		g->color = Name##_COLOR_RED;\
 		break;\
 	}\
 }\
@@ -463,12 +458,12 @@ static void Name##RBTree_insert(Name##RBTree *self, Name##RBTree *node)\
 {\
 	register Name##RBTree *n;\
 	register Name##RBTree *tmp;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_insert");\
-	CSTL_RBTREE_MAGIC(node->magic = self);\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_insert");\
+	CSTL_MAGIC(node->magic = self);\
 	n = Name##RBTree_get_root(self);\
 	if (CSTL_RBTREE_IS_NIL(n, Name)) {\
 		/* rootになる */\
-		node->color = CSTL_RBTREE_BLACK;\
+		node->color = Name##_COLOR_BLACK;\
 		Name##RBTree_set_root(self, node);\
 		return;\
 	}\
@@ -497,30 +492,30 @@ static void Name##RBTree_balance_for_erase(Name##RBTree *n, Name##RBTree *p_of_n
 	Name##RBTree *sl;\
 	Name##RBTree *sr;\
 	int c;\
-	if (n->color == CSTL_RBTREE_RED) {\
+	if (n->color == Name##_COLOR_RED) {\
 		/* case 0 nが赤 */\
-		n->color = CSTL_RBTREE_BLACK;\
+		n->color = Name##_COLOR_BLACK;\
 		return;\
 	}\
 	while (1) {\
-		if (CSTL_RBTREE_IS_ROOT(n)) {\
+		if (CSTL_RBTREE_IS_ROOT(n, Name)) {\
 			/* case 1 nがroot */\
-			n->color = CSTL_RBTREE_BLACK;\
+			n->color = Name##_COLOR_BLACK;\
 			break;\
 		}\
 		if (CSTL_RBTREE_IS_NIL(n, Name)) {\
-			assert(!(CSTL_RBTREE_IS_NIL(p_of_n->left, Name) && CSTL_RBTREE_IS_NIL(p_of_n->right, Name)) && "RBTree_balance_for_erase");\
+			CSTL_ASSERT(!(CSTL_RBTREE_IS_NIL(p_of_n->left, Name) && CSTL_RBTREE_IS_NIL(p_of_n->right, Name)) && "RBTree_balance_for_erase");\
 			p = p_of_n;\
 			s = (n == p_of_n->left) ? p_of_n->right : p_of_n->left;\
 		} else {\
 			p = n->parent;\
 			s = Name##RBTree_get_sibling(n);\
 		}\
-		assert(!CSTL_RBTREE_IS_NIL(s, Name) && "RBTree_balance_for_erase");\
-		if (s->color == CSTL_RBTREE_RED) {\
+		CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(s, Name) && "RBTree_balance_for_erase");\
+		if (s->color == Name##_COLOR_RED) {\
 			/* case 2 sが赤 */\
-			p->color = CSTL_RBTREE_RED;\
-			s->color = CSTL_RBTREE_BLACK;\
+			p->color = Name##_COLOR_RED;\
+			s->color = Name##_COLOR_BLACK;\
 			if (p->left == n) {\
 				Name##RBTree_rotate_left(p);\
 				s = p->right;\
@@ -530,49 +525,49 @@ static void Name##RBTree_balance_for_erase(Name##RBTree *n, Name##RBTree *p_of_n
 			}\
 		}\
 		/* 以下、sは黒 */\
-		assert(!CSTL_RBTREE_IS_NIL(s, Name) && "RBTree_balance_for_erase");\
+		CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(s, Name) && "RBTree_balance_for_erase");\
 		sl = s->left;\
 		sr = s->right;\
-		if (p->color == CSTL_RBTREE_BLACK && sl->color == CSTL_RBTREE_BLACK && sr->color == CSTL_RBTREE_BLACK) {\
+		if (p->color == Name##_COLOR_BLACK && sl->color == Name##_COLOR_BLACK && sr->color == Name##_COLOR_BLACK) {\
 			/* case 3 */\
-			s->color = CSTL_RBTREE_RED;\
+			s->color = Name##_COLOR_RED;\
 			n = p;\
 			/* pをnにして再調整。最悪の場合、根まで伝播 */\
 			continue;\
 		}\
-		if (p->color == CSTL_RBTREE_RED && sl->color == CSTL_RBTREE_BLACK && sr->color == CSTL_RBTREE_BLACK) {\
+		if (p->color == Name##_COLOR_RED && sl->color == Name##_COLOR_BLACK && sr->color == Name##_COLOR_BLACK) {\
 			/* case 4 */\
-			p->color = CSTL_RBTREE_BLACK;\
-			s->color = CSTL_RBTREE_RED;\
+			p->color = Name##_COLOR_BLACK;\
+			s->color = Name##_COLOR_RED;\
 			break;\
 		}\
-		if (p->left == n && sl->color == CSTL_RBTREE_RED && sr->color == CSTL_RBTREE_BLACK) {\
+		if (p->left == n && sl->color == Name##_COLOR_RED && sr->color == Name##_COLOR_BLACK) {\
 			/* case 5 nがpのleft */\
-			sl->color = CSTL_RBTREE_BLACK;\
-			s->color = CSTL_RBTREE_RED;\
+			sl->color = Name##_COLOR_BLACK;\
+			s->color = Name##_COLOR_RED;\
 			Name##RBTree_rotate_right(s);\
 			sr = s;\
 			s = sl;\
 			/* case 6 leftへ */\
-		} else if (p->right == n && sl->color == CSTL_RBTREE_BLACK && sr->color == CSTL_RBTREE_RED) {\
+		} else if (p->right == n && sl->color == Name##_COLOR_BLACK && sr->color == Name##_COLOR_RED) {\
 			/* case 5 nがpのright */\
-			sr->color = CSTL_RBTREE_BLACK;\
-			s->color = CSTL_RBTREE_RED;\
+			sr->color = Name##_COLOR_BLACK;\
+			s->color = Name##_COLOR_RED;\
 			Name##RBTree_rotate_left(s);\
 			sl = s;\
 			s = sr;\
 			/* case 6 rightへ */\
 		}\
-		if (p->left == n && sr->color == CSTL_RBTREE_RED) {\
+		if (p->left == n && sr->color == Name##_COLOR_RED) {\
 			/* case 6 left */\
-			sr->color = CSTL_RBTREE_BLACK;\
+			sr->color = Name##_COLOR_BLACK;\
 			Name##RBTree_rotate_left(p);\
-		} else if (p->right == n && sl->color == CSTL_RBTREE_RED) {\
+		} else if (p->right == n && sl->color == Name##_COLOR_RED) {\
 			/* case 6 right */\
-			sl->color = CSTL_RBTREE_BLACK;\
+			sl->color = Name##_COLOR_BLACK;\
 			Name##RBTree_rotate_right(p);\
 		} else {\
-			assert(0 && "RBTree_balance_for_erase");\
+			CSTL_ASSERT(0 && "RBTree_balance_for_erase");\
 		}\
 		c = p->color;\
 		p->color = s->color;\
@@ -585,17 +580,17 @@ static void Name##RBTree_erase(Name##RBTree *self, Name##Iterator pos)\
 {\
 	register Name##RBTree *n;\
 	register Name##RBTree *x;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_erase");\
-	assert(!CSTL_RBTREE_IS_HEAD(pos) && "RBTree_erase");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_erase");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(pos, Name) && "RBTree_erase");\
 	n = pos;\
-	assert(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_erase");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_erase");\
 	if (CSTL_RBTREE_IS_NIL(n->left, Name) && CSTL_RBTREE_IS_NIL(n->right, Name)) {\
-		if (CSTL_RBTREE_IS_ROOT(n)) {\
+		if (CSTL_RBTREE_IS_ROOT(n, Name)) {\
 			/* 最後の一つを削除 */\
 			Name##RBTree_set_root(self, (Name##RBTree *) &Name##RBTree_nil);\
 		} else {\
 			n = Name##RBTree_replace_subtree(n, (Name##RBTree *) &Name##RBTree_nil);\
-			if (n->color == CSTL_RBTREE_BLACK) {\
+			if (n->color == Name##_COLOR_BLACK) {\
 				Name##RBTree_balance_for_erase((Name##RBTree *) &Name##RBTree_nil, n->parent);\
 			}\
 		}\
@@ -603,33 +598,33 @@ static void Name##RBTree_erase(Name##RBTree *self, Name##Iterator pos)\
 	}\
 	if (CSTL_RBTREE_IS_NIL(n->left, Name)) {\
 		n = Name##RBTree_replace_subtree(n, n->right);\
-		if (n->color == CSTL_RBTREE_BLACK) {\
-			assert(!CSTL_RBTREE_IS_NIL(n->right, Name) && "RBTree_erase");\
+		if (n->color == Name##_COLOR_BLACK) {\
+			CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(n->right, Name) && "RBTree_erase");\
 			Name##RBTree_balance_for_erase(n->right, 0);\
 		}\
 		goto end;\
 	}\
 	if (CSTL_RBTREE_IS_NIL(n->right, Name)) {\
 		n = Name##RBTree_replace_subtree(n, n->left);\
-		if (n->color == CSTL_RBTREE_BLACK) {\
-			assert(!CSTL_RBTREE_IS_NIL(n->left, Name) && "RBTree_erase");\
+		if (n->color == Name##_COLOR_BLACK) {\
+			CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(n->left, Name) && "RBTree_erase");\
 			Name##RBTree_balance_for_erase(n->left, 0);\
 		}\
 		goto end;\
 	}\
-	assert(!(CSTL_RBTREE_IS_NIL(n->left, Name) || CSTL_RBTREE_IS_NIL(n->right, Name)) && "RBTree_erase");\
+	CSTL_ASSERT(!(CSTL_RBTREE_IS_NIL(n->left, Name) || CSTL_RBTREE_IS_NIL(n->right, Name)) && "RBTree_erase");\
 	x = n->left;\
 	while (!CSTL_RBTREE_IS_NIL(x->right, Name)) {\
 		x = x->right;\
 	}\
 	Name##RBTree_swap(n, x);\
 	n = Name##RBTree_replace_subtree(n, n->left);\
-	if (n->color == CSTL_RBTREE_BLACK) {\
-		assert(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_erase");\
+	if (n->color == Name##_COLOR_BLACK) {\
+		CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(n, Name) && "RBTree_erase");\
 		Name##RBTree_balance_for_erase(n->left, n->parent);\
 	}\
 end:\
-	CSTL_RBTREE_MAGIC(n->magic = 0);\
+	CSTL_MAGIC(n->magic = 0);\
 	free(n);\
 }\
 \
@@ -637,7 +632,7 @@ static Name##Iterator Name##RBTree_begin(Name##RBTree *self)\
 {\
 	register Name##RBTree *t;\
 	register Name##RBTree *tmp;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_begin");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_begin");\
 	tmp = Name##RBTree_end(self);\
 	t = Name##RBTree_get_root(self);\
 	while (!CSTL_RBTREE_IS_NIL(t, Name)) {\
@@ -649,7 +644,7 @@ static Name##Iterator Name##RBTree_begin(Name##RBTree *self)\
 \
 static Name##Iterator Name##RBTree_end(Name##RBTree *self)\
 {\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_end");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_end");\
 	return self;\
 }\
 \
@@ -657,7 +652,7 @@ static Name##Iterator Name##RBTree_rbegin(Name##RBTree *self)\
 {\
 	register Name##RBTree *t;\
 	register Name##RBTree *tmp;\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_rbegin");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_rbegin");\
 	tmp = Name##RBTree_rend(self);\
 	t = Name##RBTree_get_root(self);\
 	while (!CSTL_RBTREE_IS_NIL(t, Name)) {\
@@ -669,14 +664,14 @@ static Name##Iterator Name##RBTree_rbegin(Name##RBTree *self)\
 \
 static Name##Iterator Name##RBTree_rend(Name##RBTree *self)\
 {\
-	assert(CSTL_RBTREE_IS_HEAD(self) && "RBTree_rend");\
+	CSTL_ASSERT(CSTL_RBTREE_IS_HEAD(self, Name) && "RBTree_rend");\
 	return self;\
 }\
 \
 static Name##Iterator Name##RBTree_next(Name##Iterator pos)\
 {\
-	assert(!CSTL_RBTREE_IS_HEAD(pos) && "RBTree_next");\
-	assert(!CSTL_RBTREE_IS_NIL(pos, Name) && "RBTree_next");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(pos, Name) && "RBTree_next");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(pos, Name) && "RBTree_next");\
 	/* 下位検索 */\
 	if (!CSTL_RBTREE_IS_NIL(pos->right, Name)) {\
 		pos = pos->right;\
@@ -686,7 +681,7 @@ static Name##Iterator Name##RBTree_next(Name##Iterator pos)\
 		return pos;\
 	}\
 	/* 上位検索 */\
-	while (!CSTL_RBTREE_IS_ROOT(pos) && pos == pos->parent->right) {\
+	while (!CSTL_RBTREE_IS_ROOT(pos, Name) && pos == pos->parent->right) {\
 		pos = pos->parent;\
 	}\
 	/* 引数のposがrbegin()の時、pos->parentはend()となる。 */\
@@ -695,8 +690,8 @@ static Name##Iterator Name##RBTree_next(Name##Iterator pos)\
 \
 static Name##Iterator Name##RBTree_prev(Name##Iterator pos)\
 {\
-	assert(!CSTL_RBTREE_IS_HEAD(pos) && "RBTree_prev");\
-	assert(!CSTL_RBTREE_IS_NIL(pos, Name) && "RBTree_prev");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_HEAD(pos, Name) && "RBTree_prev");\
+	CSTL_ASSERT(!CSTL_RBTREE_IS_NIL(pos, Name) && "RBTree_prev");\
 	/* 下位検索 */\
 	if (!CSTL_RBTREE_IS_NIL(pos->left, Name)) {\
 		pos = pos->left;\
@@ -706,7 +701,7 @@ static Name##Iterator Name##RBTree_prev(Name##Iterator pos)\
 		return pos;\
 	}\
 	/* 上位検索 */\
-	while (!CSTL_RBTREE_IS_ROOT(pos) && pos == pos->parent->left) {\
+	while (!CSTL_RBTREE_IS_ROOT(pos, Name) && pos == pos->parent->left) {\
 		pos = pos->parent;\
 	}\
 	/* 引数のposがbegin()の時、pos->parentはrend()となる。 */\
@@ -734,6 +729,7 @@ size_t Name##_count(Name *self, KeyType key);\
 Name##Iterator Name##_find(Name *self, KeyType key);\
 Name##Iterator Name##_lower_bound(Name *self, KeyType key);\
 Name##Iterator Name##_upper_bound(Name *self, KeyType key);\
+void Name##_equal_range(Name *self, KeyType key, Name##Iterator *first, Name##Iterator *last);\
 Name##Iterator Name##_begin(Name *self);\
 Name##Iterator Name##_end(Name *self);\
 Name##Iterator Name##_rbegin(Name *self);\
@@ -752,7 +748,7 @@ typedef struct Name##RBTree Name##RBTree;\
 struct Name {\
 	Name##RBTree *tree;\
 	size_t size;\
-	CSTL_RBTREE_MAGIC(Name *magic;)\
+	CSTL_MAGIC(Name *magic;)\
 };\
 \
 CSTL_RBTREE_IMPLEMENT(Name, KeyType, ValueType, Compare)\
@@ -768,49 +764,49 @@ Name *Name##_new(void)\
 		return 0;\
 	}\
 	self->size = 0;\
-	CSTL_RBTREE_MAGIC(self->magic = self);\
+	CSTL_MAGIC(self->magic = self);\
 	return self;\
 }\
 \
 void Name##_delete(Name *self)\
 {\
 	if (!self) return;\
-	assert(self->magic == self && "(Set|Map)_delete");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_delete");\
 	Name##RBTree_delete(self->tree);\
-	CSTL_RBTREE_MAGIC(self->magic = 0);\
+	CSTL_MAGIC(self->magic = 0);\
 	free(self);\
 }\
 \
 void Name##_clear(Name *self)\
 {\
-	assert(self && "(Set|Map)_clear");\
-	assert(self->magic == self && "(Set|Map)_clear");\
+	CSTL_ASSERT(self && "(Set|Map)_clear");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_clear");\
 	Name##RBTree_clear(self->tree);\
 	self->size = 0;\
 }\
 \
 int Name##_empty(Name *self)\
 {\
-	assert(self && "(Set|Map)_empty");\
-	assert(self->magic == self && "(Set|Map)_empty");\
+	CSTL_ASSERT(self && "(Set|Map)_empty");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_empty");\
 	return Name##RBTree_empty(self->tree);\
 }\
 \
 size_t Name##_size(Name *self)\
 {\
-	assert(self && "(Set|Map)_size");\
-	assert(self->magic == self && "(Set|Map)_size");\
+	CSTL_ASSERT(self && "(Set|Map)_size");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_size");\
 	return self->size;\
 }\
 \
 Name##Iterator Name##_erase(Name *self, Name##Iterator pos)\
 {\
 	Name##Iterator tmp;\
-	assert(self && "(Set|Map)_erase");\
-	assert(self->magic == self && "(Set|Map)_erase");\
-	assert(pos && "(Set|Map)_erase");\
-	assert(pos != self->tree && "(Set|Map)_erase");\
-	assert(pos->magic == self->tree && "(Set|Map)_erase");\
+	CSTL_ASSERT(self && "(Set|Map)_erase");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_erase");\
+	CSTL_ASSERT(pos && "(Set|Map)_erase");\
+	CSTL_ASSERT(pos != self->tree && "(Set|Map)_erase");\
+	CSTL_ASSERT(pos->magic == self->tree && "(Set|Map)_erase");\
 	tmp = Name##_next(pos);\
 	Name##RBTree_erase(self->tree, pos);\
 	self->size--;\
@@ -820,15 +816,15 @@ Name##Iterator Name##_erase(Name *self, Name##Iterator pos)\
 Name##Iterator Name##_erase_range(Name *self, Name##Iterator first, Name##Iterator last)\
 {\
 	register Name##Iterator pos;\
-	assert(self && "(Set|Map)_erase_range");\
-	assert(self->magic == self && "(Set|Map)_erase_range");\
-	assert(first && "(Set|Map)_erase_range");\
-	assert(last && "(Set|Map)_erase_range");\
-	assert(first->magic == self->tree && "(Set|Map)_erase_range");\
-	assert(last->magic == self->tree && "(Set|Map)_erase_range");\
+	CSTL_ASSERT(self && "(Set|Map)_erase_range");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_erase_range");\
+	CSTL_ASSERT(first && "(Set|Map)_erase_range");\
+	CSTL_ASSERT(last && "(Set|Map)_erase_range");\
+	CSTL_ASSERT(first->magic == self->tree && "(Set|Map)_erase_range");\
+	CSTL_ASSERT(last->magic == self->tree && "(Set|Map)_erase_range");\
 	pos = first;\
 	while (pos != last) {\
-		assert(!Name##_empty(self) && "(Set|Map)_erase_range");\
+		CSTL_ASSERT(!Name##_empty(self) && "(Set|Map)_erase_range");\
 		pos = Name##_erase(self, pos);\
 	}\
 	return pos;\
@@ -839,12 +835,12 @@ size_t Name##_erase_key(Name *self, KeyType key)\
 	register size_t count = 0;\
 	register Name##Iterator pos;\
 	register Name##Iterator last;\
-	assert(self && "(Set|Map)_erase_key");\
-	assert(self->magic == self && "(Set|Map)_erase_key");\
+	CSTL_ASSERT(self && "(Set|Map)_erase_key");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_erase_key");\
 	pos = Name##_lower_bound(self, key);\
 	last = Name##_upper_bound(self, key);\
 	while (pos != last) {\
-		assert(!Name##_empty(self) && "(Set|Map)_erase_key");\
+		CSTL_ASSERT(!Name##_empty(self) && "(Set|Map)_erase_key");\
 		pos = Name##_erase(self, pos);\
 		count++;\
 	}\
@@ -853,71 +849,81 @@ size_t Name##_erase_key(Name *self, KeyType key)\
 \
 size_t Name##_count(Name *self, KeyType key)\
 {\
-	assert(self && "(Set|Map)_count");\
-	assert(self->magic == self && "(Set|Map)_count");\
+	CSTL_ASSERT(self && "(Set|Map)_count");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_count");\
 	return Name##RBTree_count(self->tree, key);\
 }\
 \
 Name##Iterator Name##_find(Name *self, KeyType key)\
 {\
-	assert(self && "(Set|Map)_find");\
-	assert(self->magic == self && "(Set|Map)_find");\
+	CSTL_ASSERT(self && "(Set|Map)_find");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_find");\
 	return Name##RBTree_find(self->tree, key);\
 }\
 \
 Name##Iterator Name##_lower_bound(Name *self, KeyType key)\
 {\
-	assert(self && "(Set|Map)_lower_bound");\
-	assert(self->magic == self && "(Set|Map)_lower_bound");\
+	CSTL_ASSERT(self && "(Set|Map)_lower_bound");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_lower_bound");\
 	return Name##RBTree_lower_bound(self->tree, key);\
 }\
 \
 Name##Iterator Name##_upper_bound(Name *self, KeyType key)\
 {\
-	assert(self && "(Set|Map)_upper_bound");\
-	assert(self->magic == self && "(Set|Map)_upper_bound");\
+	CSTL_ASSERT(self && "(Set|Map)_upper_bound");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_upper_bound");\
 	return Name##RBTree_upper_bound(self->tree, key);\
 }\
 \
+void Name##_equal_range(Name *self, KeyType key, Name##Iterator *first, Name##Iterator *last)\
+{\
+	CSTL_ASSERT(self && "(Set|Map)_equal_range");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_equal_range");\
+	CSTL_ASSERT(first && "(Set|Map)_equal_range");\
+	CSTL_ASSERT(last && "(Set|Map)_equal_range");\
+	*first = Name##RBTree_lower_bound(self->tree, key);\
+	*last = Name##RBTree_upper_bound(self->tree, key);\
+}\
+\
 Name##Iterator Name##_begin(Name *self)\
 {\
-	assert(self && "(Set|Map)_begin");\
-	assert(self->magic == self && "(Set|Map)_begin");\
+	CSTL_ASSERT(self && "(Set|Map)_begin");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_begin");\
 	return Name##RBTree_begin(self->tree);\
 }\
 \
 Name##Iterator Name##_end(Name *self)\
 {\
-	assert(self && "(Set|Map)_end");\
-	assert(self->magic == self && "(Set|Map)_end");\
+	CSTL_ASSERT(self && "(Set|Map)_end");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_end");\
 	return Name##RBTree_end(self->tree);\
 }\
 \
 Name##Iterator Name##_rbegin(Name *self)\
 {\
-	assert(self && "(Set|Map)_rbegin");\
-	assert(self->magic == self && "(Set|Map)_rbegin");\
+	CSTL_ASSERT(self && "(Set|Map)_rbegin");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_rbegin");\
 	return Name##RBTree_rbegin(self->tree);\
 }\
 \
 Name##Iterator Name##_rend(Name *self)\
 {\
-	assert(self && "(Set|Map)_rend");\
-	assert(self->magic == self && "(Set|Map)_rend");\
+	CSTL_ASSERT(self && "(Set|Map)_rend");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_rend");\
 	return Name##RBTree_rend(self->tree);\
 }\
 \
 Name##Iterator Name##_next(Name##Iterator pos)\
 {\
-	assert(pos && "(Set|Map)_next");\
-	assert(pos->magic && "(Set|Map)_next");\
+	CSTL_ASSERT(pos && "(Set|Map)_next");\
+	CSTL_ASSERT(pos->magic && "(Set|Map)_next");\
 	return Name##RBTree_next(pos);\
 }\
 \
 Name##Iterator Name##_prev(Name##Iterator pos)\
 {\
-	assert(pos && "(Set|Map)_prev");\
-	assert(pos->magic && "(Set|Map)_prev");\
+	CSTL_ASSERT(pos && "(Set|Map)_prev");\
+	CSTL_ASSERT(pos->magic && "(Set|Map)_prev");\
 	return Name##RBTree_prev(pos);\
 }\
 \
@@ -925,10 +931,10 @@ void Name##_swap(Name *self, Name *x)\
 {\
 	Name##RBTree *tmp_tree;\
 	size_t tmp_size;\
-	assert(self && "(Set|Map)_swap");\
-	assert(x && "(Set|Map)_swap");\
-	assert(self->magic == self && "(Set|Map)_swap");\
-	assert(x->magic == x && "(Set|Map)_swap");\
+	CSTL_ASSERT(self && "(Set|Map)_swap");\
+	CSTL_ASSERT(x && "(Set|Map)_swap");\
+	CSTL_ASSERT(self->magic == self && "(Set|Map)_swap");\
+	CSTL_ASSERT(x->magic == x && "(Set|Map)_swap");\
 	tmp_tree = self->tree;\
 	tmp_size = self->size;\
 	self->tree = x->tree;\
