@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2006-2009, KATO Noriaki
+ * Copyright (c) 2006-2010, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,29 +27,17 @@
  * \brief 文字列コンテナ
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2007-01-09
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.2/cstl/string.h $
- * $Id: string.h 278 2009-05-19 06:43:20Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.7.0/cstl/string.h $
+ * $Id: string.h 328 2010-01-24 03:49:15Z katono $
  */
 #ifndef CSTL_STRING_H_INCLUDED
 #define CSTL_STRING_H_INCLUDED
 
 #include <stdlib.h>
-#include <assert.h>
+#include <string.h>
+#include "common.h"
 #include "vector.h"
 
-#ifdef __cplusplus
-#define CSTL_STRING_BEGIN_EXTERN_C()	extern "C" {
-#define CSTL_STRING_END_EXTERN_C()		}
-#else
-#define CSTL_STRING_BEGIN_EXTERN_C()
-#define CSTL_STRING_END_EXTERN_C()
-#endif
-
-#ifdef NDEBUG
-#define CSTL_STRING_MAGIC(x)
-#else
-#define CSTL_STRING_MAGIC(x) x
-#endif
 
 #ifndef CSTL_ALGORITHM_INTERFACE
 #define CSTL_ALGORITHM_INTERFACE(Name, Type)
@@ -59,14 +47,12 @@
 #define CSTL_ALGORITHM_IMPLEMENT(Name, Type, DIRECT_ACCESS)
 #endif
 
+
 /*! 
  * \brief 検索失敗用インデックス
  */
 #define CSTL_NPOS	((size_t)-1)
 
-#define CSTL_STRING_AT(self, idx)	CSTL_VECTOR_AT((self)->data, (idx))
-#define CSTL_STRING_DEFAULT_CAPACITY	(32 - 1)
-
 
 /*! 
  * \brief インターフェイスマクロ
@@ -77,7 +63,7 @@
 #define CSTL_STRING_INTERFACE(Name, Type)	\
 typedef struct Name Name;\
 \
-CSTL_STRING_BEGIN_EXTERN_C()\
+CSTL_EXTERN_C_BEGIN()\
 Name *Name##_new(void);\
 Name *Name##_new_reserve(size_t n);\
 Name *Name##_new_assign(const Type *cstr);\
@@ -131,7 +117,7 @@ size_t Name##_find_last_not_of(Name *self, const Type *cstr, size_t idx);\
 size_t Name##_find_last_not_of_len(Name *self, const Type *chars, size_t idx, size_t chars_len);\
 size_t Name##_find_last_not_of_c(Name *self, Type c, size_t idx);\
 CSTL_ALGORITHM_INTERFACE(Name, Type)\
-CSTL_STRING_END_EXTERN_C()\
+CSTL_EXTERN_C_END()\
 
 
 /*! 
@@ -169,59 +155,62 @@ static size_t Name##_mystrlen(const Type *cstr)\
 \
 CSTL_VECTOR_INTERFACE(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_BASE(Name##_CharVector, Type)\
-CSTL_VECTOR_IMPLEMENT_PUSH_BACK(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_RESERVE(Name##_CharVector, Type)\
-CSTL_VECTOR_IMPLEMENT_SHRINK(Name##_CharVector, Type)\
-CSTL_VECTOR_IMPLEMENT_RESIZE(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_MOVE_FORWARD(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_MOVE_BACKWARD(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_INSERT_N_NO_DATA(Name##_CharVector, Type)\
+CSTL_VECTOR_IMPLEMENT_PUSH_BACK(Name##_CharVector, Type)\
+CSTL_VECTOR_IMPLEMENT_SHRINK(Name##_CharVector, Type)\
+CSTL_VECTOR_IMPLEMENT_RESIZE(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_INSERT_ARRAY(Name##_CharVector, Type)\
 CSTL_VECTOR_IMPLEMENT_ERASE(Name##_CharVector, Type)\
+CSTL_VECTOR_IMPLEMENT_SWAP(Name##_CharVector, Type)\
+\
+enum {\
+	Name##_DEFAULT_CAPACITY = 31\
+};\
 \
 /*! \
  * \brief string構造体\
+ * \
+ * vectorと同じメンバ\
  */\
 struct Name {\
-	Name##_CharVector *data;\
-	CSTL_STRING_MAGIC(Name *magic;)\
+	size_t size;\
+	size_t capacity;\
+	Type *buf;\
+	CSTL_MAGIC(Name *magic;)\
 };\
 \
 static int Name##_expand(Name *self, size_t n)\
 {\
-	return Name##_CharVector_expand(self->data, n + 1);\
+	return Name##_CharVector_expand((Name##_CharVector *) self, n + 1);\
 }\
 \
 Name *Name##_new(void)\
 {\
-	return Name##_new_reserve(CSTL_STRING_DEFAULT_CAPACITY);\
+	return Name##_new_reserve(Name##_DEFAULT_CAPACITY);\
 }\
 \
 Name *Name##_new_reserve(size_t n)\
 {\
 	Name *self;\
-	self = (Name *) malloc(sizeof(Name));\
+	self = (Name *) Name##_CharVector_new_reserve(n + 1);\
 	if (!self) return 0;\
-	self->data = Name##_CharVector_new_reserve(n + 1);\
-	if (!self->data) {\
-		free(self);\
-		return 0;\
-	}\
-	Name##_CharVector_push_back(self->data, '\0');\
-	CSTL_STRING_MAGIC(self->magic = self);\
+	Name##_CharVector_push_back((Name##_CharVector *) self, '\0');\
 	return self;\
 }\
 \
 Name *Name##_new_assign(const Type *cstr)\
 {\
-	assert(cstr && "String_new_assign");\
+	CSTL_ASSERT(cstr && "String_new_assign");\
 	return Name##_new_assign_len(cstr, CSTL_NPOS);\
 }\
 \
 Name *Name##_new_assign_len(const Type *chars, size_t chars_len)\
 {\
 	Name *self;\
-	assert(chars && "String_new_assign_len");\
+	CSTL_ASSERT(chars && "String_new_assign_len");\
 	if (chars_len == CSTL_NPOS) {\
 		chars_len = Name##_mystrlen(chars);\
 	}\
@@ -241,70 +230,68 @@ Name *Name##_new_assign_c(size_t n, Type c)\
 void Name##_delete(Name *self)\
 {\
 	if (!self) return;\
-	assert(self->magic == self && "String_delete");\
-	CSTL_STRING_MAGIC(self->magic = 0);\
-	Name##_CharVector_delete(self->data);\
-	free(self);\
+	CSTL_ASSERT(self->magic == self && "String_delete");\
+	Name##_CharVector_delete((Name##_CharVector *) self);\
 }\
 \
 void Name##_clear(Name *self)\
 {\
-	assert(self && "String_clear");\
-	assert(self->magic == self && "String_clear");\
-	CSTL_VECTOR_CLEAR(self->data);\
-	Name##_CharVector_push_back(self->data, '\0');\
+	CSTL_ASSERT(self && "String_clear");\
+	CSTL_ASSERT(self->magic == self && "String_clear");\
+	CSTL_VECTOR_CLEAR(self);\
+	Name##_CharVector_push_back((Name##_CharVector *) self, '\0');\
 }\
 \
 size_t Name##_size(Name *self)\
 {\
-	assert(self && "String_size");\
-	assert(self->magic == self && "String_size");\
-	return CSTL_VECTOR_SIZE(self->data) - 1;\
+	CSTL_ASSERT(self && "String_size");\
+	CSTL_ASSERT(self->magic == self && "String_size");\
+	return CSTL_VECTOR_SIZE(self) - 1;\
 }\
 \
 size_t Name##_length(Name *self)\
 {\
-	assert(self && "String_length");\
-	assert(self->magic == self && "String_length");\
-	return CSTL_VECTOR_SIZE(self->data) - 1;\
+	CSTL_ASSERT(self && "String_length");\
+	CSTL_ASSERT(self->magic == self && "String_length");\
+	return CSTL_VECTOR_SIZE(self) - 1;\
 }\
 \
 size_t Name##_capacity(Name *self)\
 {\
-	assert(self && "String_capacity");\
-	assert(self->magic == self && "String_capacity");\
-	return CSTL_VECTOR_CAPACITY(self->data) - 1;\
+	CSTL_ASSERT(self && "String_capacity");\
+	CSTL_ASSERT(self->magic == self && "String_capacity");\
+	return CSTL_VECTOR_CAPACITY(self) - 1;\
 }\
 \
 int Name##_reserve(Name *self, size_t n)\
 {\
-	assert(self && "String_reserve");\
-	assert(self->magic == self && "String_reserve");\
-	return Name##_CharVector_reserve(self->data, n + 1);\
+	CSTL_ASSERT(self && "String_reserve");\
+	CSTL_ASSERT(self->magic == self && "String_reserve");\
+	return Name##_CharVector_reserve((Name##_CharVector *) self, n + 1);\
 }\
 \
 void Name##_shrink(Name *self, size_t n)\
 {\
-	assert(self && "String_shrink");\
-	assert(self->magic == self && "String_shrink");\
-	Name##_CharVector_shrink(self->data, n + 1);\
+	CSTL_ASSERT(self && "String_shrink");\
+	CSTL_ASSERT(self->magic == self && "String_shrink");\
+	Name##_CharVector_shrink((Name##_CharVector *) self, n + 1);\
 }\
 \
 int Name##_empty(Name *self)\
 {\
-	assert(self && "String_empty");\
-	assert(self->magic == self && "String_empty");\
-	return CSTL_VECTOR_SIZE(self->data) == 1;\
+	CSTL_ASSERT(self && "String_empty");\
+	CSTL_ASSERT(self->magic == self && "String_empty");\
+	return CSTL_VECTOR_SIZE(self) == 1;\
 }\
 \
 int Name##_compare(Name *self, Name *x)\
 {\
 	size_t len, xlen;\
 	int ret;\
-	assert(self && "String_compare");\
-	assert(x && "String_compare");\
-	assert(self->magic == self && "String_compare");\
-	assert(x->magic == x && "String_compare");\
+	CSTL_ASSERT(self && "String_compare");\
+	CSTL_ASSERT(x && "String_compare");\
+	CSTL_ASSERT(self->magic == self && "String_compare");\
+	CSTL_ASSERT(x->magic == x && "String_compare");\
 	len = Name##_size(self);\
 	xlen = Name##_size(x);\
 	ret = Name##_mymemcmp(Name##_c_str(self), Name##_c_str(x), (len > xlen) ? xlen : len);\
@@ -321,110 +308,107 @@ int Name##_compare(Name *self, Name *x)\
 \
 Type *Name##_at(Name *self, size_t idx)\
 {\
-	assert(self && "String_at");\
-	assert(self->magic == self && "String_at");\
-	assert(Name##_size(self) > idx && "String_at");\
-	return &CSTL_VECTOR_AT(self->data, idx);\
+	CSTL_ASSERT(self && "String_at");\
+	CSTL_ASSERT(self->magic == self && "String_at");\
+	CSTL_ASSERT(Name##_size(self) > idx && "String_at");\
+	return &CSTL_VECTOR_AT(self, idx);\
 }\
 \
 const Type *Name##_c_str(Name *self)\
 {\
-	assert(self && "String_c_str");\
-	assert(self->magic == self && "String_c_str");\
-	return &CSTL_VECTOR_AT(self->data, 0);\
+	CSTL_ASSERT(self && "String_c_str");\
+	CSTL_ASSERT(self->magic == self && "String_c_str");\
+	return &CSTL_VECTOR_AT(self, 0);\
 }\
 \
 const Type *Name##_data(Name *self)\
 {\
-	assert(self && "String_data");\
-	assert(self->magic == self && "String_data");\
-	return &CSTL_VECTOR_AT(self->data, 0);\
+	CSTL_ASSERT(self && "String_data");\
+	CSTL_ASSERT(self->magic == self && "String_data");\
+	return &CSTL_VECTOR_AT(self, 0);\
 }\
 \
 Name *Name##_erase(Name *self, size_t idx, size_t len)\
 {\
 	size_t size;\
-	assert(self && "String_erase");\
-	assert(self->magic == self && "String_erase");\
-	assert(Name##_size(self) > idx && "String_erase");\
+	CSTL_ASSERT(self && "String_erase");\
+	CSTL_ASSERT(self->magic == self && "String_erase");\
+	CSTL_ASSERT(Name##_size(self) > idx && "String_erase");\
 	size = Name##_size(self);\
 	if (len > size - idx) {\
 		len = size - idx;\
 	}\
-	Name##_CharVector_erase(self->data, idx, len);\
+	Name##_CharVector_erase((Name##_CharVector *) self, idx, len);\
 	return self;\
 }\
 \
 void Name##_pop_back(Name *self)\
 {\
 	size_t size;\
-	assert(self && "String_pop_back");\
-	assert(self->magic == self && "String_pop_back");\
-	assert(!Name##_empty(self) && "String_pop_back");\
+	CSTL_ASSERT(self && "String_pop_back");\
+	CSTL_ASSERT(self->magic == self && "String_pop_back");\
+	CSTL_ASSERT(!Name##_empty(self) && "String_pop_back");\
 	size = Name##_size(self);\
-	Name##_CharVector_erase(self->data, size - 1, 1);\
+	Name##_CharVector_erase((Name##_CharVector *) self, size - 1, 1);\
 }\
 \
 int Name##_resize(Name *self, size_t n, Type c)\
 {\
 	size_t num;\
-	assert(self && "String_resize");\
-	assert(self->magic == self && "String_resize");\
-	num = CSTL_VECTOR_SIZE(self->data) - 1;\
-	if (!Name##_CharVector_resize(self->data, n + 1, c)) {\
+	CSTL_ASSERT(self && "String_resize");\
+	CSTL_ASSERT(self->magic == self && "String_resize");\
+	num = CSTL_VECTOR_SIZE(self) - 1;\
+	if (!Name##_CharVector_resize((Name##_CharVector *) self, n + 1, c)) {\
 		return 0;\
 	}\
-	if (num < CSTL_VECTOR_SIZE(self->data)) {\
+	if (num < CSTL_VECTOR_SIZE(self)) {\
 		/* '\0'を上書き */\
-		CSTL_VECTOR_AT(self->data, num) = c;\
+		CSTL_VECTOR_AT(self, num) = c;\
 	}\
-	CSTL_VECTOR_AT(self->data, CSTL_VECTOR_SIZE(self->data) - 1) = '\0';\
+	CSTL_VECTOR_AT(self, CSTL_VECTOR_SIZE(self) - 1) = '\0';\
 	return 1;\
 }\
 \
 void Name##_swap(Name *self, Name *x)\
 {\
-	Name##_CharVector *tmp_data;\
-	assert(self && "String_swap");\
-	assert(x && "String_swap");\
-	assert(self->magic == self && "String_swap");\
-	assert(x->magic == x && "String_swap");\
-	tmp_data = self->data;\
-	self->data = x->data;\
-	x->data = tmp_data;\
+	CSTL_ASSERT(self && "String_swap");\
+	CSTL_ASSERT(x && "String_swap");\
+	CSTL_ASSERT(self->magic == self && "String_swap");\
+	CSTL_ASSERT(x->magic == x && "String_swap");\
+	Name##_CharVector_swap((Name##_CharVector *) self, (Name##_CharVector *) x);\
 }\
 \
 Name *Name##_assign(Name *self, const Type *cstr)\
 {\
-	assert(self && "String_assign");\
-	assert(self->magic == self && "String_assign");\
-	assert(cstr && "String_assign");\
+	CSTL_ASSERT(self && "String_assign");\
+	CSTL_ASSERT(self->magic == self && "String_assign");\
+	CSTL_ASSERT(cstr && "String_assign");\
 	return Name##_assign_len(self, cstr, CSTL_NPOS);\
 }\
 \
 Name *Name##_assign_len(Name *self, const Type *chars, size_t chars_len)\
 {\
-	assert(self && "String_assign_len");\
-	assert(self->magic == self && "String_assign_len");\
-	assert(chars && "String_assign_len");\
+	CSTL_ASSERT(self && "String_assign_len");\
+	CSTL_ASSERT(self->magic == self && "String_assign_len");\
+	CSTL_ASSERT(chars && "String_assign_len");\
 	if (chars_len == CSTL_NPOS) {\
 		chars_len = Name##_mystrlen(chars);\
 	}\
 	/* NOTE: charsがself内の文字列の場合、chars_lenはString_size()より小さいはずなので\
 	 * Vector_expand()は許容量拡張せず、内部バッファのアドレスが変わることはない。 */\
-	if (!Name##_CharVector_expand(self->data, chars_len + 1)) {\
+	if (!Name##_CharVector_expand((Name##_CharVector *) self, chars_len + 1)) {\
 		return 0;\
 	}\
-	CSTL_VECTOR_CLEAR(self->data);\
-	Name##_CharVector_insert_array(self->data, 0, chars, chars_len);\
-	Name##_CharVector_push_back(self->data, '\0');\
+	CSTL_VECTOR_CLEAR(self);\
+	Name##_CharVector_insert_array((Name##_CharVector *) self, 0, chars, chars_len);\
+	Name##_CharVector_push_back((Name##_CharVector *) self, '\0');\
 	return self;\
 }\
 \
 Name *Name##_assign_c(Name *self, size_t n, Type c)\
 {\
-	assert(self && "String_assign_c");\
-	assert(self->magic == self && "String_assign_c");\
+	CSTL_ASSERT(self && "String_assign_c");\
+	CSTL_ASSERT(self->magic == self && "String_assign_c");\
 	if (!Name##_expand(self, n)) {\
 		return 0;\
 	}\
@@ -434,17 +418,17 @@ Name *Name##_assign_c(Name *self, size_t n, Type c)\
 \
 Name *Name##_append(Name *self, const Type *cstr)\
 {\
-	assert(self && "String_append");\
-	assert(self->magic == self && "String_append");\
-	assert(cstr && "String_append");\
+	CSTL_ASSERT(self && "String_append");\
+	CSTL_ASSERT(self->magic == self && "String_append");\
+	CSTL_ASSERT(cstr && "String_append");\
 	return Name##_append_len(self, cstr, CSTL_NPOS);\
 }\
 \
 Name *Name##_append_len(Name *self, const Type *chars, size_t chars_len)\
 {\
-	assert(self && "String_append_len");\
-	assert(self->magic == self && "String_append_len");\
-	assert(chars && "String_append_len");\
+	CSTL_ASSERT(self && "String_append_len");\
+	CSTL_ASSERT(self->magic == self && "String_append_len");\
+	CSTL_ASSERT(chars && "String_append_len");\
 	if (chars_len == CSTL_NPOS) {\
 		chars_len = Name##_mystrlen(chars);\
 	}\
@@ -453,91 +437,75 @@ Name *Name##_append_len(Name *self, const Type *chars, size_t chars_len)\
 \
 Name *Name##_append_c(Name *self, size_t n, Type c)\
 {\
-	assert(self && "String_append_c");\
-	assert(self->magic == self && "String_append_c");\
+	CSTL_ASSERT(self && "String_append_c");\
+	CSTL_ASSERT(self->magic == self && "String_append_c");\
 	return Name##_insert_c(self, Name##_size(self), n, c);\
 }\
 \
 Name *Name##_push_back(Name *self, Type c)\
 {\
-	assert(self && "String_push_back");\
-	assert(self->magic == self && "String_push_back");\
+	CSTL_ASSERT(self && "String_push_back");\
+	CSTL_ASSERT(self->magic == self && "String_push_back");\
 	return Name##_insert_c(self, Name##_size(self), 1, c);\
 }\
 \
 static int Name##_insert_n_no_data(Name *self, size_t idx, size_t n)\
 {\
-	return Name##_CharVector_insert_n_no_data(self->data, idx, n);\
+	return Name##_CharVector_insert_n_no_data((Name##_CharVector *) self, idx, n);\
 }\
 \
 Name *Name##_insert(Name *self, size_t idx, const Type *cstr)\
 {\
-	assert(self && "String_insert");\
-	assert(self->magic == self && "String_insert");\
-	assert(Name##_size(self) >= idx && "String_insert");\
-	assert(cstr && "String_insert");\
+	CSTL_ASSERT(self && "String_insert");\
+	CSTL_ASSERT(self->magic == self && "String_insert");\
+	CSTL_ASSERT(Name##_size(self) >= idx && "String_insert");\
+	CSTL_ASSERT(cstr && "String_insert");\
 	return Name##_insert_len(self, idx, cstr, CSTL_NPOS);\
 }\
 \
 Name *Name##_insert_len(Name *self, size_t idx, const Type *chars, size_t chars_len)\
 {\
-	register size_t i;\
 	size_t size;\
-	assert(self && "String_insert_len");\
-	assert(self->magic == self && "String_insert_len");\
-	assert(chars && "String_insert_len");\
+	CSTL_ASSERT(self && "String_insert_len");\
+	CSTL_ASSERT(self->magic == self && "String_insert_len");\
+	CSTL_ASSERT(chars && "String_insert_len");\
 	size = Name##_size(self);\
-	assert(size >= idx && "String_insert_len");\
+	CSTL_ASSERT(size >= idx && "String_insert_len");\
 	if (chars_len == CSTL_NPOS) {\
 		chars_len = Name##_mystrlen(chars);\
 	}\
 	if (Name##_c_str(self) <= chars && chars < Name##_c_str(self) + size) {\
 		if (Name##_capacity(self) < size + chars_len) {\
 			/* NOTE: charsがself内の文字列の場合、許容量拡張で内部バッファのアドレスが変わって\
-			 * charsが読めなくなる可能性があるためtmpにコピーして置き換える。 */\
+			 * charsが読めなくなる可能性があるためtmpにコピーする。 */\
 			Type *tmp = (Type *) malloc(sizeof(Type) * chars_len);\
 			if (!tmp) return 0;\
-			for (i = 0; i < chars_len; i++) {\
-				tmp[i] = chars[i];\
-			}\
-			chars = tmp;\
+			memcpy(tmp, chars, sizeof(Type) * chars_len);\
 			if (!Name##_insert_n_no_data(self, idx, chars_len)) {\
 				free(tmp);\
 				return 0;\
 			}\
-			for (i = 0; i < chars_len; i++) {\
-				CSTL_STRING_AT(self, idx + i) = chars[i];\
-			}\
+			memcpy(&CSTL_VECTOR_AT(self, idx), tmp, sizeof(Type) * chars_len);\
 			free(tmp);\
 		} else {\
 			/* charsがself内の文字列だが、許容量拡張はされない */\
 			/* insert_n_no_data()は必ず真を返す */\
 			Name##_insert_n_no_data(self, idx, chars_len);\
-			if (&CSTL_STRING_AT(self, idx) <= chars) {\
-				for (i = 0; i < chars_len; i++) {\
-					CSTL_STRING_AT(self, idx + i) = chars[chars_len + i];\
-				}\
-			} else if (chars < &CSTL_STRING_AT(self, idx) && &CSTL_STRING_AT(self, idx) < chars + chars_len) {\
-				size_t k = &CSTL_STRING_AT(self, idx) - chars;\
-				for (i = 0; i < k; i++) {\
-					CSTL_STRING_AT(self, idx + i) = chars[i];\
-				}\
-				for (i = 0; i < chars_len - k; i++) {\
-					CSTL_STRING_AT(self, idx + k + i) = CSTL_STRING_AT(self, idx + chars_len + i);\
-				}\
+			if (&CSTL_VECTOR_AT(self, idx) <= chars) {\
+				memcpy(&CSTL_VECTOR_AT(self, idx), &chars[chars_len], sizeof(Type) * chars_len);\
+			} else if (chars < &CSTL_VECTOR_AT(self, idx) && &CSTL_VECTOR_AT(self, idx) < chars + chars_len) {\
+				size_t k = &CSTL_VECTOR_AT(self, idx) - chars;\
+				memcpy(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * k);\
+				memcpy(&CSTL_VECTOR_AT(self, idx + k), &CSTL_VECTOR_AT(self, idx + chars_len), sizeof(Type) * (chars_len - k));\
 			} else {\
-				for (i = 0; i < chars_len; i++) {\
-					CSTL_STRING_AT(self, idx + i) = chars[i];\
-				}\
+				memcpy(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * chars_len);\
 			}\
 		}\
 	} else {\
 		if (!Name##_insert_n_no_data(self, idx, chars_len)) {\
 			return 0;\
 		}\
-		for (i = 0; i < chars_len; i++) {\
-			CSTL_STRING_AT(self, idx + i) = chars[i];\
-		}\
+		memcpy(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * chars_len);\
 	}\
 	return self;\
 }\
@@ -545,36 +513,35 @@ Name *Name##_insert_len(Name *self, size_t idx, const Type *chars, size_t chars_
 Name *Name##_insert_c(Name *self, size_t idx, size_t n, Type c)\
 {\
 	register size_t i;\
-	assert(self && "String_insert_c");\
-	assert(self->magic == self && "String_insert_c");\
-	assert(Name##_size(self) >= idx && "String_insert_c");\
+	CSTL_ASSERT(self && "String_insert_c");\
+	CSTL_ASSERT(self->magic == self && "String_insert_c");\
+	CSTL_ASSERT(Name##_size(self) >= idx && "String_insert_c");\
 	if (!Name##_insert_n_no_data(self, idx, n)) {\
 		return 0;\
 	}\
 	for (i = 0; i < n; i++) {\
-		CSTL_STRING_AT(self, idx + i) = c;\
+		CSTL_VECTOR_AT(self, idx + i) = c;\
 	}\
 	return self;\
 }\
 \
 Name *Name##_replace(Name *self, size_t idx, size_t len, const Type *cstr)\
 {\
-	assert(self && "String_replace");\
-	assert(self->magic == self && "String_replace");\
-	assert(cstr && "String_replace");\
-	assert(Name##_size(self) >= idx && "String_replace");\
+	CSTL_ASSERT(self && "String_replace");\
+	CSTL_ASSERT(self->magic == self && "String_replace");\
+	CSTL_ASSERT(cstr && "String_replace");\
+	CSTL_ASSERT(Name##_size(self) >= idx && "String_replace");\
 	return Name##_replace_len(self, idx, len, cstr, CSTL_NPOS);\
 }\
 \
 Name *Name##_replace_len(Name *self, size_t idx, size_t len, const Type *chars, size_t chars_len)\
 {\
-	register size_t i;\
 	size_t size;\
-	assert(self && "String_replace_len");\
-	assert(self->magic == self && "String_replace_len");\
-	assert(chars && "String_replace_len");\
+	CSTL_ASSERT(self && "String_replace_len");\
+	CSTL_ASSERT(self->magic == self && "String_replace_len");\
+	CSTL_ASSERT(chars && "String_replace_len");\
 	size = Name##_size(self);\
-	assert(size >= idx && "String_replace_len");\
+	CSTL_ASSERT(size >= idx && "String_replace_len");\
 	if (len > size - idx) {\
 		len = size - idx;\
 	}\
@@ -585,15 +552,7 @@ Name *Name##_replace_len(Name *self, size_t idx, size_t len, const Type *chars,
 		/* charsがself内の文字列 */\
 		if (chars_len <= len) {\
 			/* 拡張必要なし */\
-			if (&CSTL_STRING_AT(self, idx) <= chars) {\
-				for (i = 0; i < chars_len; i++) {\
-					CSTL_STRING_AT(self, idx + i) = chars[i];\
-				}\
-			} else {\
-				for (i = chars_len; i > 0; i--) {\
-					CSTL_STRING_AT(self, idx + i - 1) = chars[i - 1];\
-				}\
-			}\
+			memmove(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * chars_len);\
 			if (chars_len != len) {\
 				Name##_erase(self, idx + chars_len, len - chars_len);\
 			}\
@@ -601,43 +560,32 @@ Name *Name##_replace_len(Name *self, size_t idx, size_t len, const Type *chars,
 			if (Name##_capacity(self) < size + (chars_len - len)) {\
 				/* 拡張必要あり */\
 				/* NOTE: charsがself内の文字列の場合、許容量拡張で内部バッファのアドレスが変わって\
-				 * charsが読めなくなる可能性があるためtmpにコピーして置き換える。 */\
+				 * charsが読めなくなる可能性があるためtmpにコピーする。 */\
 				Type *tmp = (Type *) malloc(sizeof(Type) * chars_len);\
 				if (!tmp) return 0;\
-				for (i = 0; i < chars_len; i++) {\
-					tmp[i] = chars[i];\
-				}\
-				chars = tmp;\
+				memcpy(tmp, chars, sizeof(Type) * chars_len);\
 				if (!Name##_expand(self, size + (chars_len - len))) {\
 					free(tmp);\
 					return 0;\
 				}\
-				for (i = 0; i < len; i++) {\
-					CSTL_STRING_AT(self, idx + i) = chars[i];\
-				}\
-				Name##_insert_len(self, idx + len, &chars[len], chars_len - len);\
+				memcpy(&CSTL_VECTOR_AT(self, idx), tmp, sizeof(Type) * len);\
+				Name##_insert_len(self, idx + len, &tmp[len], chars_len - len);\
 				free(tmp);\
 			} else {\
 				/* charsがself内の文字列だが、許容量拡張はされない */\
-				if (&CSTL_STRING_AT(self, idx) <= chars) {\
-					for (i = 0; i < len; i++) {\
-						CSTL_STRING_AT(self, idx + i) = chars[i];\
-					}\
+				if (&CSTL_VECTOR_AT(self, idx) <= chars) {\
+					memmove(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * len);\
 					Name##_insert_len(self, idx + len, &chars[len], chars_len - len);\
 				} else {\
 					Name##_insert_len(self, idx + len, &chars[len], chars_len - len);\
-					for (i = len; i > 0; i--) {\
-						CSTL_STRING_AT(self, idx + i - 1) = chars[i - 1];\
-					}\
+					memmove(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * len);\
 				}\
 			}\
 		}\
 	} else {\
 		if (chars_len <= len) {\
 			/* 拡張必要なし */\
-			for (i = 0; i < chars_len; i++) {\
-				CSTL_STRING_AT(self, idx + i) = chars[i];\
-			}\
+			memcpy(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * chars_len);\
 			if (chars_len != len) {\
 				Name##_erase(self, idx + chars_len, len - chars_len);\
 			}\
@@ -646,9 +594,7 @@ Name *Name##_replace_len(Name *self, size_t idx, size_t len, const Type *chars,
 			if (!Name##_expand(self, Name##_size(self) + (chars_len - len))) {\
 				return 0;\
 			}\
-			for (i = 0; i < len; i++) {\
-				CSTL_STRING_AT(self, idx + i) = chars[i];\
-			}\
+			memcpy(&CSTL_VECTOR_AT(self, idx), chars, sizeof(Type) * len);\
 			Name##_insert_len(self, idx + len, &chars[len], chars_len - len);\
 		}\
 	}\
@@ -659,9 +605,9 @@ Name *Name##_replace_c(Name *self, size_t idx, size_t len, size_t n, Type c)\
 {\
 	register size_t i;\
 	size_t size;\
-	assert(self && "String_replace_c");\
-	assert(self->magic == self && "String_replace_c");\
-	assert(Name##_size(self) >= idx && "String_replace_c");\
+	CSTL_ASSERT(self && "String_replace_c");\
+	CSTL_ASSERT(self->magic == self && "String_replace_c");\
+	CSTL_ASSERT(Name##_size(self) >= idx && "String_replace_c");\
 	size = Name##_size(self);\
 	if (len > size - idx) {\
 		len = size - idx;\
@@ -669,7 +615,7 @@ Name *Name##_replace_c(Name *self, size_t idx, size_t len, size_t n, Type c)\
 	if (n <= len) {\
 		/* 拡張必要なし */\
 		for (i = 0; i < n; i++) {\
-			CSTL_STRING_AT(self, idx + i) = c;\
+			CSTL_VECTOR_AT(self, idx + i) = c;\
 		}\
 		if (n != len) {\
 			Name##_erase(self, idx + n, len - n);\
@@ -680,7 +626,7 @@ Name *Name##_replace_c(Name *self, size_t idx, size_t len, size_t n, Type c)\
 			return 0;\
 		}\
 		for (i = 0; i < n; i++) {\
-			CSTL_STRING_AT(self, idx + i) = c;\
+			CSTL_VECTOR_AT(self, idx + i) = c;\
 		}\
 	}\
 	return self;\
@@ -722,9 +668,9 @@ static size_t Name##_brute_force_search_r(const Type *str, size_t str_len, const
 \
 size_t Name##_find(Name *self, const Type *cstr, size_t idx)\
 {\
-	assert(self && "String_find");\
-	assert(self->magic == self && "String_find");\
-	assert(cstr && "String_find");\
+	CSTL_ASSERT(self && "String_find");\
+	CSTL_ASSERT(self->magic == self && "String_find");\
+	CSTL_ASSERT(cstr && "String_find");\
 	return Name##_find_len(self, cstr, idx, CSTL_NPOS);\
 }\
 \
@@ -732,9 +678,9 @@ size_t Name##_find_len(Name *self, const Type *chars, size_t idx, size_t chars_l
 {\
 	size_t i;\
 	size_t size;\
-	assert(self && "String_find_len");\
-	assert(self->magic == self && "String_find_len");\
-	assert(chars && "String_find_len");\
+	CSTL_ASSERT(self && "String_find_len");\
+	CSTL_ASSERT(self->magic == self && "String_find_len");\
+	CSTL_ASSERT(chars && "String_find_len");\
 	if (chars_len == CSTL_NPOS) {\
 		chars_len = Name##_mystrlen(chars);\
 	}\
@@ -750,25 +696,25 @@ size_t Name##_find_len(Name *self, const Type *chars, size_t idx, size_t chars_l
 \
 size_t Name##_find_c(Name *self, Type c, size_t idx)\
 {\
-	assert(self && "String_find_c");\
-	assert(self->magic == self && "String_find_c");\
+	CSTL_ASSERT(self && "String_find_c");\
+	CSTL_ASSERT(self->magic == self && "String_find_c");\
 	return Name##_find_len(self, &c, idx, 1);\
 }\
 \
 size_t Name##_rfind(Name *self, const Type *cstr, size_t idx)\
 {\
-	assert(self && "String_rfind");\
-	assert(self->magic == self && "String_rfind");\
-	assert(cstr && "String_rfind");\
+	CSTL_ASSERT(self && "String_rfind");\
+	CSTL_ASSERT(self->magic == self && "String_rfind");\
+	CSTL_ASSERT(cstr && "String_rfind");\
 	return Name##_rfind_len(self, cstr, idx, CSTL_NPOS);\
 }\
 \
 size_t Name##_rfind_len(Name *self, const Type *chars, size_t idx, size_t chars_len)\
 {\
 	size_t size;\
-	assert(self && "String_rfind_len");\
-	assert(self->magic == self && "String_rfind_len");\
-	assert(chars && "String_rfind_len");\
+	CSTL_ASSERT(self && "String_rfind_len");\
+	CSTL_ASSERT(self->magic == self && "String_rfind_len");\
+	CSTL_ASSERT(chars && "String_rfind_len");\
 	if (chars_len == CSTL_NPOS) {\
 		chars_len = Name##_mystrlen(chars);\
 	}\
@@ -784,16 +730,16 @@ size_t Name##_rfind_len(Name *self, const Type *chars, size_t idx, size_t chars_
 \
 size_t Name##_rfind_c(Name *self, Type c, size_t idx)\
 {\
-	assert(self && "String_rfind_c");\
-	assert(self->magic == self && "String_rfind_c");\
+	CSTL_ASSERT(self && "String_rfind_c");\
+	CSTL_ASSERT(self->magic == self && "String_rfind_c");\
 	return Name##_rfind_len(self, &c, idx, 1);\
 }\
 \
 size_t Name##_find_first_of(Name *self, const Type *cstr, size_t idx)\
 {\
-	assert(self && "String_find_first_of");\
-	assert(self->magic == self && "String_find_first_of");\
-	assert(cstr && "String_find_first_of");\
+	CSTL_ASSERT(self && "String_find_first_of");\
+	CSTL_ASSERT(self->magic == self && "String_find_first_of");\
+	CSTL_ASSERT(cstr && "String_find_first_of");\
 	return Name##_find_first_of_len(self, cstr, idx, CSTL_NPOS);\
 }\
 \
@@ -802,9 +748,9 @@ size_t Name##_find_first_of_len(Name *self, const Type *chars, size_t idx, size_
 	register size_t i, j;\
 	size_t size;\
 	const Type *p;\
-	assert(self && "String_find_first_of_len");\
-	assert(self->magic == self && "String_find_first_of_len");\
-	assert(chars && "String_find_first_of_len");\
+	CSTL_ASSERT(self && "String_find_first_of_len");\
+	CSTL_ASSERT(self->magic == self && "String_find_first_of_len");\
+	CSTL_ASSERT(chars && "String_find_first_of_len");\
 	size = Name##_size(self);\
 	if (size <= idx) return CSTL_NPOS;\
 	if (chars_len == CSTL_NPOS) {\
@@ -823,16 +769,16 @@ size_t Name##_find_first_of_len(Name *self, const Type *chars, size_t idx, size_
 \
 size_t Name##_find_first_of_c(Name *self, Type c, size_t idx)\
 {\
-	assert(self && "String_find_first_of_c");\
-	assert(self->magic == self && "String_find_first_of_c");\
+	CSTL_ASSERT(self && "String_find_first_of_c");\
+	CSTL_ASSERT(self->magic == self && "String_find_first_of_c");\
 	return Name##_find_first_of_len(self, &c, idx, 1);\
 }\
 \
 size_t Name##_find_last_of(Name *self, const Type *cstr, size_t idx)\
 {\
-	assert(self && "String_find_last_of");\
-	assert(self->magic == self && "String_find_last_of");\
-	assert(cstr && "String_find_last_of");\
+	CSTL_ASSERT(self && "String_find_last_of");\
+	CSTL_ASSERT(self->magic == self && "String_find_last_of");\
+	CSTL_ASSERT(cstr && "String_find_last_of");\
 	return Name##_find_last_of_len(self, cstr, idx, CSTL_NPOS);\
 }\
 \
@@ -841,9 +787,9 @@ size_t Name##_find_last_of_len(Name *self, const Type *chars, size_t idx, size_t
 	register size_t i, j;\
 	size_t size;\
 	const Type *p;\
-	assert(self && "String_find_last_of_len");\
-	assert(self->magic == self && "String_find_last_of_len");\
-	assert(chars && "String_find_last_of_len");\
+	CSTL_ASSERT(self && "String_find_last_of_len");\
+	CSTL_ASSERT(self->magic == self && "String_find_last_of_len");\
+	CSTL_ASSERT(chars && "String_find_last_of_len");\
 	size = Name##_size(self);\
 	if (!size) {\
 		return CSTL_NPOS;\
@@ -867,16 +813,16 @@ size_t Name##_find_last_of_len(Name *self, const Type *chars, size_t idx, size_t
 \
 size_t Name##_find_last_of_c(Name *self, Type c, size_t idx)\
 {\
-	assert(self && "String_find_last_of_c");\
-	assert(self->magic == self && "String_find_last_of_c");\
+	CSTL_ASSERT(self && "String_find_last_of_c");\
+	CSTL_ASSERT(self->magic == self && "String_find_last_of_c");\
 	return Name##_find_last_of_len(self, &c, idx, 1);\
 }\
 \
 size_t Name##_find_first_not_of(Name *self, const Type *cstr, size_t idx)\
 {\
-	assert(self && "String_find_first_not_of");\
-	assert(self->magic == self && "String_find_first_not_of");\
-	assert(cstr && "String_find_first_not_of");\
+	CSTL_ASSERT(self && "String_find_first_not_of");\
+	CSTL_ASSERT(self->magic == self && "String_find_first_not_of");\
+	CSTL_ASSERT(cstr && "String_find_first_not_of");\
 	return Name##_find_first_not_of_len(self, cstr, idx, CSTL_NPOS);\
 }\
 \
@@ -885,9 +831,9 @@ size_t Name##_find_first_not_of_len(Name *self, const Type *chars, size_t idx, s
 	register size_t i, j;\
 	size_t size;\
 	const Type *p;\
-	assert(self && "String_find_first_not_of_len");\
-	assert(self->magic == self && "String_find_first_not_of_len");\
-	assert(chars && "String_find_first_not_of_len");\
+	CSTL_ASSERT(self && "String_find_first_not_of_len");\
+	CSTL_ASSERT(self->magic == self && "String_find_first_not_of_len");\
+	CSTL_ASSERT(chars && "String_find_first_not_of_len");\
 	size = Name##_size(self);\
 	if (size <= idx) return CSTL_NPOS;\
 	if (chars_len == CSTL_NPOS) {\
@@ -911,16 +857,16 @@ size_t Name##_find_first_not_of_len(Name *self, const Type *chars, size_t idx, s
 \
 size_t Name##_find_first_not_of_c(Name *self, Type c, size_t idx)\
 {\
-	assert(self && "String_find_first_not_of_c");\
-	assert(self->magic == self && "String_find_first_not_of_c");\
+	CSTL_ASSERT(self && "String_find_first_not_of_c");\
+	CSTL_ASSERT(self->magic == self && "String_find_first_not_of_c");\
 	return Name##_find_first_not_of_len(self, &c, idx, 1);\
 }\
 \
 size_t Name##_find_last_not_of(Name *self, const Type *cstr, size_t idx)\
 {\
-	assert(self && "String_find_last_not_of");\
-	assert(self->magic == self && "String_find_last_not_of");\
-	assert(cstr && "String_find_last_not_of");\
+	CSTL_ASSERT(self && "String_find_last_not_of");\
+	CSTL_ASSERT(self->magic == self && "String_find_last_not_of");\
+	CSTL_ASSERT(cstr && "String_find_last_not_of");\
 	return Name##_find_last_not_of_len(self, cstr, idx, CSTL_NPOS);\
 }\
 \
@@ -929,9 +875,9 @@ size_t Name##_find_last_not_of_len(Name *self, const Type *chars, size_t idx, si
 	register size_t i, j;\
 	size_t size;\
 	const Type *p;\
-	assert(self && "String_find_last_not_of_len");\
-	assert(self->magic == self && "String_find_last_not_of_len");\
-	assert(chars && "String_find_last_not_of_len");\
+	CSTL_ASSERT(self && "String_find_last_not_of_len");\
+	CSTL_ASSERT(self->magic == self && "String_find_last_not_of_len");\
+	CSTL_ASSERT(chars && "String_find_last_not_of_len");\
 	size = Name##_size(self);\
 	if (!size) {\
 		return CSTL_NPOS;\
@@ -960,12 +906,12 @@ size_t Name##_find_last_not_of_len(Name *self, const Type *chars, size_t idx, si
 \
 size_t Name##_find_last_not_of_c(Name *self, Type c, size_t idx)\
 {\
-	assert(self && "String_find_last_not_of_c");\
-	assert(self->magic == self && "String_find_last_not_of_c");\
+	CSTL_ASSERT(self && "String_find_last_not_of_c");\
+	CSTL_ASSERT(self->magic == self && "String_find_last_not_of_c");\
 	return Name##_find_last_not_of_len(self, &c, idx, 1);\
 }\
 \
-CSTL_ALGORITHM_IMPLEMENT(Name, Type, CSTL_STRING_AT)\
+CSTL_ALGORITHM_IMPLEMENT(Name, Type, CSTL_VECTOR_AT)\
 
 
 #endif /* CSTL_STRING_H_INCLUDED */
