@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2006-2009, KATO Noriaki
+ * Copyright (c) 2006-2010, KATO Noriaki
  * All rights reserved.
  * 
  * Redistribution and use in source and binary forms, with or without
@@ -27,28 +27,16 @@
  * \brief vectorコンテナ
  * \author KATO Noriaki <katono@users.sourceforge.jp>
  * \date 2006-11-29
- * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.6.2/cstl/vector.h $
- * $Id: vector.h 278 2009-05-19 06:43:20Z katono $
+ * $URL: http://svn.sourceforge.jp/svnroot/cstl/tags/rel-0.7.0/cstl/vector.h $
+ * $Id: vector.h 328 2010-01-24 03:49:15Z katono $
  */
 #ifndef CSTL_VECTOR_H_INCLUDED
 #define CSTL_VECTOR_H_INCLUDED
 
 #include <stdlib.h>
-#include <assert.h>
+#include <string.h>
+#include "common.h"
 
-#ifdef __cplusplus
-#define CSTL_VECTOR_BEGIN_EXTERN_C()	extern "C" {
-#define CSTL_VECTOR_END_EXTERN_C()		}
-#else
-#define CSTL_VECTOR_BEGIN_EXTERN_C()
-#define CSTL_VECTOR_END_EXTERN_C()
-#endif
-
-#ifdef NDEBUG
-#define CSTL_VECTOR_MAGIC(x)
-#else
-#define CSTL_VECTOR_MAGIC(x) x
-#endif
 
 #ifndef CSTL_ALGORITHM_INTERFACE
 #define CSTL_ALGORITHM_INTERFACE(Name, Type)
@@ -76,11 +64,12 @@
 #define CSTL_VECTOR_INTERFACE(Name, Type)	\
 typedef struct Name Name;\
 \
-CSTL_VECTOR_BEGIN_EXTERN_C()\
+CSTL_EXTERN_C_BEGIN()\
 Name *Name##_new(void);\
 Name *Name##_new_reserve(size_t n);\
 void Name##_delete(Name *self);\
 int Name##_push_back(Name *self, Type data);\
+int Name##_push_back_ref(Name *self, Type const *data);\
 void Name##_pop_back(Name *self);\
 size_t Name##_size(Name *self);\
 size_t Name##_capacity(Name *self);\
@@ -93,21 +82,19 @@ Type *Name##_at(Name *self, size_t idx);\
 Type *Name##_front(Name *self);\
 Type *Name##_back(Name *self);\
 int Name##_insert(Name *self, size_t idx, Type data);\
+int Name##_insert_ref(Name *self, size_t idx, Type const *data);\
 int Name##_insert_n(Name *self, size_t idx, size_t n, Type data);\
+int Name##_insert_n_ref(Name *self, size_t idx, size_t n, Type const *data);\
 int Name##_insert_array(Name *self, size_t idx, Type const *data, size_t n);\
 int Name##_insert_range(Name *self, size_t idx, Name *x, size_t xidx, size_t n);\
 void Name##_erase(Name *self, size_t idx, size_t n);\
 void Name##_swap(Name *self, Name *x);\
 CSTL_ALGORITHM_INTERFACE(Name, Type)\
-CSTL_VECTOR_END_EXTERN_C()\
+CSTL_EXTERN_C_END()\
 
 
 
 #define CSTL_VECTOR_IMPLEMENT_BASE(Name, Type)	\
-static int Name##_expand(Name *self, size_t size);\
-static void Name##_move_forward(Name *self, size_t first, size_t last, size_t n);\
-static void Name##_move_backward(Name *self, size_t first, size_t last, size_t n);\
-static int Name##_insert_n_no_data(Name *self, size_t idx, size_t n);\
 /*! \
  * \brief vector構造体\
  */\
@@ -115,7 +102,7 @@ struct Name {\
 	size_t size;\
 	size_t capacity;\
 	Type *buf;\
-	CSTL_VECTOR_MAGIC(Name *magic;)\
+	CSTL_MAGIC(Name *magic;)\
 };\
 \
 Name *Name##_new(void)\
@@ -126,7 +113,7 @@ Name *Name##_new(void)\
 	self->capacity = 0;\
 	self->size = 0;\
 	self->buf = 0;\
-	CSTL_VECTOR_MAGIC(self->magic = self);\
+	CSTL_MAGIC(self->magic = self);\
 	return self;\
 }\
 \
@@ -145,8 +132,8 @@ Name *Name##_new_reserve(size_t n)\
 void Name##_delete(Name *self)\
 {\
 	if (!self) return;\
-	assert(self->magic == self && "Vector_delete");\
-	CSTL_VECTOR_MAGIC(self->magic = 0);\
+	CSTL_ASSERT(self->magic == self && "Vector_delete");\
+	CSTL_MAGIC(self->magic = 0);\
 	free(self->buf);\
 	free(self);\
 }\
@@ -156,10 +143,18 @@ void Name##_delete(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_PUSH_BACK(Name, Type)	\
 int Name##_push_back(Name *self, Type data)\
 {\
-	assert(self && "Vector_push_back");\
-	assert(self->magic == self && "Vector_push_back");\
+	CSTL_ASSERT(self && "Vector_push_back");\
+	CSTL_ASSERT(self->magic == self && "Vector_push_back");\
+	return Name##_push_back_ref(self, &data);\
+}\
+\
+int Name##_push_back_ref(Name *self, Type const *data)\
+{\
+	CSTL_ASSERT(self && "Vector_push_back_ref");\
+	CSTL_ASSERT(self->magic == self && "Vector_push_back_ref");\
+	CSTL_ASSERT(data && "Vector_push_back_ref");\
 	if (!Name##_expand(self, CSTL_VECTOR_SIZE(self) + 1)) return 0;\
-	self->buf[self->size] = data;\
+	self->buf[self->size] = *data;\
 	self->size++;\
 	return 1;\
 }\
@@ -168,9 +163,9 @@ int Name##_push_back(Name *self, Type data)\
 #define CSTL_VECTOR_IMPLEMENT_POP_BACK(Name, Type)	\
 void Name##_pop_back(Name *self)\
 {\
-	assert(self && "Vector_pop_back");\
-	assert(self->magic == self && "Vector_pop_back");\
-	assert(!CSTL_VECTOR_EMPTY(self) && "Vector_pop_back");\
+	CSTL_ASSERT(self && "Vector_pop_back");\
+	CSTL_ASSERT(self->magic == self && "Vector_pop_back");\
+	CSTL_ASSERT(!CSTL_VECTOR_EMPTY(self) && "Vector_pop_back");\
 	self->size--;\
 }\
 \
@@ -178,8 +173,8 @@ void Name##_pop_back(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_SIZE(Name, Type)	\
 size_t Name##_size(Name *self)\
 {\
-	assert(self && "Vector_size");\
-	assert(self->magic == self && "Vector_size");\
+	CSTL_ASSERT(self && "Vector_size");\
+	CSTL_ASSERT(self->magic == self && "Vector_size");\
 	return CSTL_VECTOR_SIZE(self);\
 }\
 \
@@ -187,8 +182,8 @@ size_t Name##_size(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_CAPACITY(Name, Type)	\
 size_t Name##_capacity(Name *self)\
 {\
-	assert(self && "Vector_capacity");\
-	assert(self->magic == self && "Vector_capacity");\
+	CSTL_ASSERT(self && "Vector_capacity");\
+	CSTL_ASSERT(self->magic == self && "Vector_capacity");\
 	return CSTL_VECTOR_CAPACITY(self);\
 }\
 \
@@ -196,8 +191,8 @@ size_t Name##_capacity(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_EMPTY(Name, Type)	\
 int Name##_empty(Name *self)\
 {\
-	assert(self && "Vector_empty");\
-	assert(self->magic == self && "Vector_empty");\
+	CSTL_ASSERT(self && "Vector_empty");\
+	CSTL_ASSERT(self->magic == self && "Vector_empty");\
 	return CSTL_VECTOR_EMPTY(self);\
 }\
 \
@@ -205,8 +200,8 @@ int Name##_empty(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_CLEAR(Name, Type)	\
 void Name##_clear(Name *self)\
 {\
-	assert(self && "Vector_clear");\
-	assert(self->magic == self && "Vector_clear");\
+	CSTL_ASSERT(self && "Vector_clear");\
+	CSTL_ASSERT(self->magic == self && "Vector_clear");\
 	CSTL_VECTOR_CLEAR(self);\
 }\
 \
@@ -223,9 +218,13 @@ static int Name##_expand(Name *self, size_t size)\
 int Name##_reserve(Name *self, size_t n)\
 {\
 	Type *newbuf;\
-	assert(self && "Vector_reserve");\
-	assert(self->magic == self && "Vector_reserve");\
+	CSTL_ASSERT(self && "Vector_reserve");\
+	CSTL_ASSERT(self->magic == self && "Vector_reserve");\
 	if (n <= CSTL_VECTOR_CAPACITY(self)) return 1;\
+	if (n > ((size_t) -1) / sizeof(Type)) {\
+		/* sizeof(Type)*n がオーバーフローする */\
+		return 0;\
+	}\
 	newbuf = (Type *) realloc(self->buf, sizeof(Type) * n);\
 	if (!newbuf) return 0;\
 	self->buf = newbuf;\
@@ -238,18 +237,20 @@ int Name##_reserve(Name *self, size_t n)\
 void Name##_shrink(Name *self, size_t n)\
 {\
 	Type *newbuf;\
-	assert(self && "Vector_shrink");\
-	assert(self->magic == self && "Vector_shrink");\
+	CSTL_ASSERT(self && "Vector_shrink");\
+	CSTL_ASSERT(self->magic == self && "Vector_shrink");\
 	if (n >= CSTL_VECTOR_CAPACITY(self)) return;\
 	if (n < CSTL_VECTOR_SIZE(self)) {\
 		n = CSTL_VECTOR_SIZE(self);\
 	}\
 	self->capacity = n;\
-	newbuf = (Type *) realloc(self->buf, sizeof(Type) * n);\
 	if (!n) {\
-		/* nが0ならreallocはfreeと等価 */\
+		free(self->buf);\
 		self->buf = 0;\
-	} else if (newbuf) {\
+		return;\
+	}\
+	newbuf = (Type *) realloc(self->buf, sizeof(Type) * n);\
+	if (newbuf) {\
 		self->buf = newbuf;\
 	}\
 }\
@@ -259,8 +260,8 @@ void Name##_shrink(Name *self, size_t n)\
 int Name##_resize(Name *self, size_t n, Type data)\
 {\
 	size_t size;\
-	assert(self && "Vector_resize");\
-	assert(self->magic == self && "Vector_resize");\
+	CSTL_ASSERT(self && "Vector_resize");\
+	CSTL_ASSERT(self->magic == self && "Vector_resize");\
 	size = CSTL_VECTOR_SIZE(self);\
 	if (size >= n) {\
 		self->size = n;\
@@ -280,9 +281,9 @@ int Name##_resize(Name *self, size_t n, Type data)\
 #define CSTL_VECTOR_IMPLEMENT_AT(Name, Type)	\
 Type *Name##_at(Name *self, size_t idx)\
 {\
-	assert(self && "Vector_at");\
-	assert(self->magic == self && "Vector_at");\
-	assert(CSTL_VECTOR_SIZE(self) > idx && "Vector_at");\
+	CSTL_ASSERT(self && "Vector_at");\
+	CSTL_ASSERT(self->magic == self && "Vector_at");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) > idx && "Vector_at");\
 	return &CSTL_VECTOR_AT(self, idx);\
 }\
 \
@@ -290,9 +291,9 @@ Type *Name##_at(Name *self, size_t idx)\
 #define CSTL_VECTOR_IMPLEMENT_FRONT(Name, Type)	\
 Type *Name##_front(Name *self)\
 {\
-	assert(self && "Vector_front");\
-	assert(self->magic == self && "Vector_front");\
-	assert(!CSTL_VECTOR_EMPTY(self) && "Vector_front");\
+	CSTL_ASSERT(self && "Vector_front");\
+	CSTL_ASSERT(self->magic == self && "Vector_front");\
+	CSTL_ASSERT(!CSTL_VECTOR_EMPTY(self) && "Vector_front");\
 	return &self->buf[0];\
 }\
 \
@@ -300,9 +301,9 @@ Type *Name##_front(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_BACK(Name, Type)	\
 Type *Name##_back(Name *self)\
 {\
-	assert(self && "Vector_back");\
-	assert(self->magic == self && "Vector_back");\
-	assert(!CSTL_VECTOR_EMPTY(self) && "Vector_back");\
+	CSTL_ASSERT(self && "Vector_back");\
+	CSTL_ASSERT(self->magic == self && "Vector_back");\
+	CSTL_ASSERT(!CSTL_VECTOR_EMPTY(self) && "Vector_back");\
 	return &self->buf[self->size - 1];\
 }\
 \
@@ -310,20 +311,14 @@ Type *Name##_back(Name *self)\
 #define CSTL_VECTOR_IMPLEMENT_MOVE_BACKWARD(Name, Type)	\
 static void Name##_move_backward(Name *self, size_t first, size_t last, size_t n)\
 {\
-	register size_t i;\
-	for (i = last; i > first; i--) {\
-		self->buf[i - 1 + n] = self->buf[i - 1];\
-	}\
+	memmove(&self->buf[first + n], &self->buf[first], sizeof(Type) * (last - first));\
 }\
 \
 
 #define CSTL_VECTOR_IMPLEMENT_MOVE_FORWARD(Name, Type)	\
 static void Name##_move_forward(Name *self, size_t first, size_t last, size_t n)\
 {\
-	register size_t i;\
-	for (i = first; i < last; i++) {\
-		self->buf[i - n] = self->buf[i];\
-	}\
+	memmove(&self->buf[first - n], &self->buf[first], sizeof(Type) * (last - first));\
 }\
 \
 
@@ -340,25 +335,43 @@ static int Name##_insert_n_no_data(Name *self, size_t idx, size_t n)\
 #define CSTL_VECTOR_IMPLEMENT_INSERT(Name, Type)	\
 int Name##_insert(Name *self, size_t idx, Type data)\
 {\
-	assert(self && "Vector_insert");\
-	assert(self->magic == self && "Vector_insert");\
-	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert");\
-	return Name##_insert_array(self, idx, &data, 1);\
+	CSTL_ASSERT(self && "Vector_insert");\
+	CSTL_ASSERT(self->magic == self && "Vector_insert");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert");\
+	return Name##_insert_n_ref(self, idx, 1, &data);\
+}\
+\
+int Name##_insert_ref(Name *self, size_t idx, Type const *data)\
+{\
+	CSTL_ASSERT(self && "Vector_insert_ref");\
+	CSTL_ASSERT(self->magic == self && "Vector_insert_ref");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_ref");\
+	CSTL_ASSERT(data && "Vector_insert_ref");\
+	return Name##_insert_n_ref(self, idx, 1, data);\
 }\
 \
 
 #define CSTL_VECTOR_IMPLEMENT_INSERT_N(Name, Type)	\
 int Name##_insert_n(Name *self, size_t idx, size_t n, Type data)\
+{\
+	CSTL_ASSERT(self && "Vector_insert_n");\
+	CSTL_ASSERT(self->magic == self && "Vector_insert_n");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_n");\
+	return Name##_insert_n_ref(self, idx, n, &data);\
+}\
+\
+int Name##_insert_n_ref(Name *self, size_t idx, size_t n, Type const *data)\
 {\
 	register size_t i;\
-	assert(self && "Vector_insert_n");\
-	assert(self->magic == self && "Vector_insert_n");\
-	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_n");\
+	CSTL_ASSERT(self && "Vector_insert_n_ref");\
+	CSTL_ASSERT(self->magic == self && "Vector_insert_n_ref");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_n_ref");\
+	CSTL_ASSERT(data && "Vector_insert_n_ref");\
 	if (!Name##_insert_n_no_data(self, idx, n)) {\
 		return 0;\
 	}\
 	for (i = 0; i < n; i++) {\
-		self->buf[idx + i] = data;\
+		self->buf[idx + i] = *data;\
 	}\
 	return 1;\
 }\
@@ -367,17 +380,14 @@ int Name##_insert_n(Name *self, size_t idx, size_t n, Type data)\
 #define CSTL_VECTOR_IMPLEMENT_INSERT_ARRAY(Name, Type)	\
 int Name##_insert_array(Name *self, size_t idx, Type const *data, size_t n)\
 {\
-	register size_t i;\
-	assert(self && "Vector_insert_array");\
-	assert(self->magic == self && "Vector_insert_array");\
-	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_array");\
-	assert(data && "Vector_insert_array");\
+	CSTL_ASSERT(self && "Vector_insert_array");\
+	CSTL_ASSERT(self->magic == self && "Vector_insert_array");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_array");\
+	CSTL_ASSERT(data && "Vector_insert_array");\
 	if (!Name##_insert_n_no_data(self, idx, n)) {\
 		return 0;\
 	}\
-	for (i = 0; i < n; i++) {\
-		self->buf[idx + i] = data[i];\
-	}\
+	memcpy(&self->buf[idx], data, sizeof(Type) * n);\
 	return 1;\
 }\
 \
@@ -385,39 +395,28 @@ int Name##_insert_array(Name *self, size_t idx, Type const *data, size_t n)\
 #define CSTL_VECTOR_IMPLEMENT_INSERT_RANGE(Name, Type)	\
 int Name##_insert_range(Name *self, size_t idx, Name *x, size_t xidx, size_t n)\
 {\
-	register size_t i;\
-	assert(self && "Vector_insert_range");\
-	assert(self->magic == self && "Vector_insert_range");\
-	assert(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_range");\
-	assert(x && "Vector_insert_range");\
-	assert(x->magic == x && "Vector_insert_range");\
-	assert(CSTL_VECTOR_SIZE(x) >= xidx + n && "Vector_insert_range");\
-	assert(CSTL_VECTOR_SIZE(x) >= n && "Vector_insert_range");\
-	assert(CSTL_VECTOR_SIZE(x) > xidx && "Vector_insert_range");\
+	CSTL_ASSERT(self && "Vector_insert_range");\
+	CSTL_ASSERT(self->magic == self && "Vector_insert_range");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx && "Vector_insert_range");\
+	CSTL_ASSERT(x && "Vector_insert_range");\
+	CSTL_ASSERT(x->magic == x && "Vector_insert_range");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(x) >= xidx + n && "Vector_insert_range");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(x) >= n && "Vector_insert_range");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(x) > xidx && "Vector_insert_range");\
 	if (!Name##_insert_n_no_data(self, idx, n)) {\
 		return 0;\
 	}\
 	if (self == x) {\
 		if (idx <= xidx) {\
-			for (i = 0; i < n; i++) {\
-				CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(self, xidx + n + i);\
-			}\
+			memcpy(&CSTL_VECTOR_AT(self, idx), &CSTL_VECTOR_AT(self, xidx + n), sizeof(Type) * n);\
 		} else if (xidx < idx && idx < xidx + n) {\
-			for (i = 0; i < idx - xidx; i++) {\
-				CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(self, xidx + i);\
-			}\
-			for (i = 0; i < n - (idx - xidx); i++) {\
-				CSTL_VECTOR_AT(self, idx + (idx - xidx) + i) = CSTL_VECTOR_AT(self, idx + n + i);\
-			}\
+			memcpy(&CSTL_VECTOR_AT(self, idx), &CSTL_VECTOR_AT(self, xidx), sizeof(Type) * (idx - xidx));\
+			memcpy(&CSTL_VECTOR_AT(self, idx + (idx - xidx)), &CSTL_VECTOR_AT(self, idx + n), sizeof(Type) * (n - (idx - xidx)));\
 		} else {\
-			for (i = 0; i < n; i++) {\
-				CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(self, xidx + i);\
-			}\
+			memcpy(&CSTL_VECTOR_AT(self, idx), &CSTL_VECTOR_AT(self, xidx), sizeof(Type) * n);\
 		}\
 	} else {\
-		for (i = 0; i < n; i++) {\
-			CSTL_VECTOR_AT(self, idx + i) = CSTL_VECTOR_AT(x, xidx + i);\
-		}\
+		memcpy(&CSTL_VECTOR_AT(self, idx), &CSTL_VECTOR_AT(x, xidx), sizeof(Type) * n);\
 	}\
 	return 1;\
 }\
@@ -426,11 +425,11 @@ int Name##_insert_range(Name *self, size_t idx, Name *x, size_t xidx, size_t n)\
 #define CSTL_VECTOR_IMPLEMENT_ERASE(Name, Type)	\
 void Name##_erase(Name *self, size_t idx, size_t n)\
 {\
-	assert(self && "Vector_erase");\
-	assert(self->magic == self && "Vector_erase");\
-	assert(CSTL_VECTOR_SIZE(self) >= idx + n && "Vector_erase");\
-	assert(CSTL_VECTOR_SIZE(self) >= n && "Vector_erase");\
-	assert(CSTL_VECTOR_SIZE(self) > idx && "Vector_erase");\
+	CSTL_ASSERT(self && "Vector_erase");\
+	CSTL_ASSERT(self->magic == self && "Vector_erase");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= idx + n && "Vector_erase");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) >= n && "Vector_erase");\
+	CSTL_ASSERT(CSTL_VECTOR_SIZE(self) > idx && "Vector_erase");\
 	Name##_move_forward(self, idx + n, self->size, n);\
 	self->size -= n;\
 }\
@@ -442,10 +441,10 @@ void Name##_swap(Name *self, Name *x)\
 	size_t tmp_size;\
 	size_t tmp_capacity;\
 	Type *tmp_buf;\
-	assert(self && "Vector_swap");\
-	assert(x && "Vector_swap");\
-	assert(self->magic == self && "Vector_swap");\
-	assert(x->magic == x && "Vector_swap");\
+	CSTL_ASSERT(self && "Vector_swap");\
+	CSTL_ASSERT(x && "Vector_swap");\
+	CSTL_ASSERT(self->magic == self && "Vector_swap");\
+	CSTL_ASSERT(x->magic == x && "Vector_swap");\
 	tmp_size = self->size;\
 	tmp_capacity = self->capacity;\
 	tmp_buf = self->buf;\
@@ -467,21 +466,21 @@ void Name##_swap(Name *self, Name *x)\
  */
 #define CSTL_VECTOR_IMPLEMENT(Name, Type)	\
 CSTL_VECTOR_IMPLEMENT_BASE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_RESERVE(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_MOVE_FORWARD(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_MOVE_BACKWARD(Name, Type)\
+CSTL_VECTOR_IMPLEMENT_INSERT_N_NO_DATA(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_PUSH_BACK(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_POP_BACK(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_SIZE(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_CAPACITY(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_EMPTY(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_CLEAR(Name, Type)\
-CSTL_VECTOR_IMPLEMENT_RESERVE(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_SHRINK(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_RESIZE(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_AT(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_FRONT(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_BACK(Name, Type)\
-CSTL_VECTOR_IMPLEMENT_MOVE_FORWARD(Name, Type)\
-CSTL_VECTOR_IMPLEMENT_MOVE_BACKWARD(Name, Type)\
-CSTL_VECTOR_IMPLEMENT_INSERT_N_NO_DATA(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_INSERT(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_INSERT_N(Name, Type)\
 CSTL_VECTOR_IMPLEMENT_INSERT_ARRAY(Name, Type)\
