@@ -1,8 +1,8 @@
 #include <stdio.h>
 #include <cstl/set.h>
 
-SET_INTERFACE(IntSet, int)
-SET_IMPLEMENT(IntSet, int, SIMPLE_CMP, ASC)
+CSTL_SET_INTERFACE(IntSet, int)
+CSTL_SET_IMPLEMENT(IntSet, int, CSTL_LESS)
 
 int main(void)
 {
@@ -21,7 +21,7 @@ int main(void)
 	IntSet_insert(x, 7, NULL);
 
 	for (pos = IntSet_begin(x); pos != IntSet_end(x); pos = IntSet_next(pos)) {
-		printf("%d ", IntSet_key(pos));
+		printf("%d ", *IntSet_data(pos));
 	}
 	printf("\n");
 
@@ -36,14 +36,14 @@ int main(void)
 	IntSet_erase_key(x, 5);
 
 	for (pos = IntSet_begin(x); pos != IntSet_end(x); pos = IntSet_next(pos)) {
-		printf("%d ", IntSet_key(pos));
+		printf("%d ", *IntSet_data(pos));
 	}
 	printf("\n");
 
-	printf("lower_bound(5): %d\n", IntSet_key(IntSet_lower_bound(x, 5)));
-	printf("upper_bound(5): %d\n", IntSet_key(IntSet_upper_bound(x, 5)));
-	printf("lower_bound(6): %d\n", IntSet_key(IntSet_lower_bound(x, 6)));
-	printf("upper_bound(6): %d\n", IntSet_key(IntSet_upper_bound(x, 6)));
+	printf("lower_bound(5): %d\n", *IntSet_data(IntSet_lower_bound(x, 5)));
+	printf("upper_bound(5): %d\n", *IntSet_data(IntSet_upper_bound(x, 5)));
+	printf("lower_bound(6): %d\n", *IntSet_data(IntSet_lower_bound(x, 6)));
+	printf("upper_bound(6): %d\n", *IntSet_data(IntSet_upper_bound(x, 6)));
 	IntSet_delete(x);
 	return 0;
 }
